@@ -1,5 +1,5 @@
-import sys
-sys.path.append("..")
+#!/usr/bin/env python
+import iam_sympy_example
 
 from sympy import Derivative, Symbol, Function, exp, Rational, log, \
     dsolve
