@@ -1,13 +1,16 @@
+#!/usr/bin/env python
+import iam_sympy_example
+
 """
 Go to the sympy root directory and execute:
 
-$ python examples/fem_test.py 
-1/60 0 -1/360 0 -1/90 -1/360 
-0 4/45 0 2/45 2/45 -1/90 
--1/360 0 1/60 -1/90 0 -1/360 
-0 2/45 -1/90 4/45 2/45 0 
--1/90 2/45 0 2/45 4/45 0 
--1/360 -1/90 -1/360 0 0 1/60 
+$ python examples/fem_test.py
+[  1/60,     0, -1/360,     0, -1/90, -1/360]
+[     0,  4/45,      0,  2/45,  2/45,  -1/90]
+[-1/360,     0,   1/60, -1/90,     0, -1/360]
+[     0,  2/45,  -1/90,  4/45,  2/45,      0]
+[ -1/90,  2/45,      0,  2/45,  4/45,      0]
+[-1/360, -1/90, -1/360,     0,     0,   1/60]
 """
 
 from fem import *
@@ -33,4 +36,4 @@ for i in range(0, fe.nbf()):
        J[j,i] = t.integrate(integrands)
 
 
-print J
+pprint(J)
