@@ -1,3 +1,6 @@
+#!/usr/bin/env python
+import iam_sympy_example
+
 """
 Plotting Examples
 
@@ -6,8 +9,7 @@ to use plotting. It is included with Python 2.5 and later.
 
 Suggested Usage:    python -i plotting.py
 """
-import sys
-sys.path.append("..")
+
 
 from sympy import symbols
 from sympy import Plot
@@ -134,15 +136,15 @@ if __name__ == "__main__":
             from sympy.plotting.plot_interval import PlotInterval
             from pyglet.gl import glBegin, glColor3f
             from pyglet.gl import glVertex3f, glEnd, GL_LINES
-            
+
             def draw_gradient_vectors(f, iu, iv):
                 """
                 Create a function which draws vectors
                 representing the gradient of f.
                 """
                 dx, dy, dz = f.diff(x), f.diff(y), 0
-                FF = lambdify( [x,y,f],    [x,y] )
-                FG = lambdify( [dx,dy,dz], [x,y] )
+                FF = lambdify( [x,y], [x,y,f] )
+                FG = lambdify( [x,y], [dx,dy,dz] )
                 iu.v_steps /= 5
                 iv.v_steps /= 5
                 Gvl = list(list([FF(u, v), FG(u, v)]
