@@ -1,10 +1,10 @@
-import sys
-sys.path.append("..")
+#!/usr/bin/env python
+import iam_sympy_example
 
 from sympy import *
 
-x=Symbol("x") 
-y=Symbol("y") 
+x=Symbol("x")
+y=Symbol("y")
 
 
 pprint( x**x )
@@ -31,6 +31,6 @@ print '\n'
 pprint( sqrt(sqrt(exp(x))) )
 print '\n'
 
-pprint( (1/cos(x)).series(x,10) )
+pprint( (1/cos(x)).series(x,0,10) )
 print '\n'
 
