@@ -1,13 +1,12 @@
-import sys
-sys.path.append("..")
-sys.path.append(".")
+#!/usr/bin/env python
+import iam_sympy_example
 
 from sympy import sqrt,exp,log,Symbol,oo,Rational,sin,cos,limit,I,pi,Mul
 from sympy.core import basic
 from sympy.printing import print_pygame, pprint
 
-x=Symbol("x") 
-y=Symbol("y") 
+x=Symbol("x")
+y=Symbol("y")
 
 expressions = (
     x**x,
@@ -19,7 +18,7 @@ expressions = (
     sin(x**2+exp(x)),
     sqrt(exp(x)),
 
-    #print (1/cos(x)).series(x,10)
+    #print (1/cos(x)).series(x,0,10)
 )
 
 print "sympy print:"
