@@ -1,19 +1,17 @@
+#!/usr/bin/env python
+import iam_sympy_example
+
 #a sandbox, for playing with new ideas and algorithms
 
-
-import sys
-sys.path.append("..")
-sys.path.append(".")
-
 from sympy import exp,log,Symbol,oo,Rational,sin,cos,limit,I,pi,Mul
 from sympy import Integral
 from sympy import limits
 from sympy.core import basic
 limits.debug=True
 
-x=Symbol("x") 
-y=Symbol("y") 
-w=Symbol("w") 
+x=Symbol("x")
+y=Symbol("y")
+w=Symbol("w")
 
 #import pdb
 #pdb.run('print limit(sin(x)/x,x,oo)')
