@@ -1,6 +1,6 @@
 
 from products import product, Product
-from summations import Sum
+from summations import sum, Sum
 from sums_products import Sum2
 from gosper import normal, gosper
-from utilities import filter_roots, nni_roots
\ No newline at end of file
+from utilities import filter_roots, nni_roots
