@@ -1,15 +1,17 @@
 """Core module. Provides the basic operations needed in sympy.
 """
 
-from basic import Basic, S
+from basic import Basic, S, C, sympify
 from symbol import Symbol, Wild, symbols, var
 from numbers import Number, Real, Rational, Integer
 from power import Pow
 from mul import Mul
 from add import Add
-from relational import Equality, Inequality, Unequality, StrictInequality
+from relational import Rel, Eq, Ne, Lt, Le, Gt, Ge, \
+    Equality, Inequality, Unequality, StrictInequality
+from multidimensional import vectorize
 from function import Lambda, WildFunction, Derivative, diff, FunctionClass, \
-    Function
+    Function, expand
 from interval import Interval
 
 # set repr output to pretty output:
@@ -19,4 +21,3 @@ Basic.set_repr_level(1)
 for _n, _cls in Basic.singleton.items():
     exec '%s = _cls()' % (_n)
 
-sympify = Basic.sympify
