@@ -1,60 +1,46 @@
 """
-NOTE: this doctest is for now obsoleted interface.
-
-There are different types of functions:
+There are two types of functions:
 1) defined function like exp or sin that has a name and body
    (in the sense that function can be evaluated).
     e = exp
 2) undefined function with a name but no body. Undefined
-  functions can be defined using Symbol class as follows:
-    f = Symbol('f', function=True)
-  (the result will be Function instance)
-  or
+  functions can be defined using a Function class as follows:
     f = Function('f')
-3) anonymous function or lambda function that has no name
-   but has body with dummy variables. An anonymous function
+  (the result will be Function instance)
+3) this isn't implemented yet: anonymous function or lambda function that has
+no name but has body with dummy variables. An anonymous function
    object creation examples:
     f = Lambda(x, exp(x)*x)
     f = Lambda(exp(x)*x)  # free symbols in the expression define the number of arguments
     f = exp * Lambda(x,x)
-4) composition of functions, inverse functions
+4) isn't implemented yet: composition of functions, like (sin+cos)(x), this
+works in sympycore, but needs to be ported back to SymPy.
 
-One can perform certain operations with functions, the
-result will be a lambda function. Allowed operations are
-addition and multiplication. Function multiplication is
-elementise ie (f*g)(x) is equivalent to f(x) * g(x).
-Multiplication by a number is equivalent to multiplication
-by a constant function.
-Composition of functions is achived via Composition class@
-Eg
 
-  f+Composition(2,exp,sin) -> lambda _x: f(x)+2*exp(sin(x))
-
-In the above functions did not have arguments, then
-it is said that functions are in unevaluated form.
-When calling a function with arguments, then we get
-an instance of the function value. Eg
-
-  exp(1) -> 1
-  (2*f)(x)  -> 2 * f(x)
-  Lambda(x, exp(x)*x)(y) -> exp(y)*y
-
-One can construct undefined function values from Symbol
-object:
-  f = Symbol('f')
-  fx = f(x)
-  fx.func -> Function('f')
-  fx[:] -> (Symbol('x'),)
-As seen above, function values are Apply instances and
-have attributes .func and [:].
+Example:
+    >>> from sympy import *
+    >>> f = Function("f")
+    >>> x = Symbol("x")
+    >>> f(x)
+    f(x)
+    >>> f(x).func
+    <class 'sympy.core.function.f'>
+    >>> f(x).args
+    (x,)
 """
 
-from basic import Basic, Singleton, Atom, cache_it, S
-from basic_methods import BasicType, MetaBasicMeths
-from methods import ArithMeths, NoRelMeths, RelMeths
+from basic import Basic, Singleton, Atom, S, C, sympify
+from basic import BasicType, BasicMeta
+from methods import ArithMeths, RelMeths
 from operations import AssocOp
+from cache import cacheit
+from itertools import repeat
+from numbers import Rational,Integer
+from symbol import Symbol
+from add    import Add
+from multidimensional import vectorize
 
-class FunctionClass(MetaBasicMeths):
+class FunctionClass(BasicMeta):
     """
     Base class for function classes. FunctionClass is a subclass of type.
 
@@ -92,13 +78,28 @@ class Function(Basic, ArithMeths, RelMeths):
 
     __metaclass__ = FunctionClass
 
+    is_Function = True
+
     precedence = Basic.Apply_precedence
 
     nargs = None
 
-    @cache_it
+    @vectorize(1)
+    @cacheit
     def __new__(cls, *args, **options):
-        if cls is Function or cls is Function:
+        # NOTE: this __new__ is twofold:
+        #
+        # 1 -- it can create another *class*, which can then be instantiated by
+        #      itself e.g. Function('f') creates a new class f(Function)
+        #
+        # 2 -- on the other hand, we instantiate -- that is we create an
+        #      *instance* of a class created earlier in 1.
+        #
+        # So please keep, both (1) and (2) in mind.
+
+        # (1) create new function class
+        #     UC: Function('f')
+        if cls is Function:
             #when user writes Function("f"), do an equivalent of:
             #taking the whole class Function(...):
             #and rename the Function to "f" and return f, thus:
@@ -115,18 +116,15 @@ class Function(Basic, ArithMeths, RelMeths):
                 print args
                 print type(args[0])
                 raise Exception("You need to specify exactly one string")
-        args = map(Basic.sympify, args)
+
+        # (2) create new instance of a class created in (1)
+        #     UC: Function('f')(x)
+        #     UC: sin(x)
+        args = map(sympify, args)
         # these lines should be refactored
-        if "nargs" in options:
-            del options["nargs"]
-        if "dummy" in options:
-            del options["dummy"]
-        if "comparable" in options:
-            del options["comparable"]
-        if "noncommutative" in options:
-            del options["noncommutative"]
-        if "commutative" in options:
-            del options["commutative"]
+        for opt in ["nargs", "dummy", "comparable", "noncommutative", "commutative"]:
+            if opt in options:
+                del options[opt]
         # up to here.
         r = cls.canonize(*args, **options)
         if isinstance(r, Basic):
@@ -137,10 +135,6 @@ class Function(Basic, ArithMeths, RelMeths):
             args = (r,)
         return Basic.__new__(cls, *args, **options)
 
-    @property
-    def is_comparable(self):
-        return True
-
     @property
     def is_commutative(self):
         return True
@@ -160,15 +154,15 @@ class Function(Basic, ArithMeths, RelMeths):
 
         @classmethod
         def canonize(cls, arg):
-            if isinstance(arg, Basic.NaN):
+            if arg is S.NaN:
                 return S.NaN
-            if isinstance(arg, Basic.Zero): return S.One
+            if arg is S.Zero: return S.One
             if arg.is_positive: return S.One
             if arg.is_negative: return S.NegativeOne
-            if isinstance(arg, Basic.Mul):
+            if isinstance(arg, C.Mul):
                 coeff, terms = arg.as_coeff_terms()
-                if not isinstance(coeff, Basic.One):
-                    return cls(coeff) * cls(Basic.Mul(*terms))
+                if coeff is not S.One:
+                    return cls(coeff) * cls(C.Mul(*terms))
 
         """
         return
@@ -182,25 +176,25 @@ class Function(Basic, ArithMeths, RelMeths):
             return new
         elif isinstance(old, FunctionClass) and isinstance(new, FunctionClass):
             if old == self.func and old.nargs == new.nargs:
-                return new(*self[:])
-        obj = self.func._eval_apply_subs(*(self[:] + (old,) + (new,)))
+                return new(*self.args[:])
+        obj = self.func._eval_apply_subs(*(self.args[:] + (old,) + (new,)))
         if obj is not None:
             return obj
         return Basic._seq_subs(self, old, new)
 
     def _eval_expand_basic(self, *args):
-        return self
+        return None
 
     def _eval_evalf(self):
-        obj = self.func._eval_apply_evalf(*self[:])
+        obj = self.func._eval_apply_evalf(*self.args[:])
         if obj is None:
             return self
         return obj
 
     def _eval_is_comparable(self):
-        if isinstance(self.func, DefinedFunction):
+        if self.is_Function:
             r = True
-            for s in self:
+            for s in self.args:
                 c = s.is_comparable
                 if c is None: return
                 if not c: r = False
@@ -208,22 +202,19 @@ class Function(Basic, ArithMeths, RelMeths):
         return
 
     def _eval_derivative(self, s):
-        # Apply(f(x), x).diff(s) -> x.diff(s) * f.fdiff(1)(s)
+        # f(x).diff(s) -> x.diff(s) * f.fdiff(1)(s)
         i = 0
         l = []
-        r = Basic.Zero()
-        for a in self:
+        r = S.Zero
+        for a in self.args:
             i += 1
             da = a.diff(s)
-            if isinstance(da, Basic.Zero):
+            if da is S.Zero:
                 continue
             if isinstance(self.func, FunctionClass):
                 df = self.fdiff(i)
                 l.append(df * da)
-            #else:
-            #    df = self.func.fdiff(i)
-            #    l.append(Apply(df,*self[:]) * da)
-        return Basic.Add(*l)
+        return Add(*l)
 
     def _eval_is_commutative(self):
         r = True
@@ -250,56 +241,59 @@ class Function(Basic, ArithMeths, RelMeths):
             return True
 
     def as_base_exp(self):
-        return self, Basic.One()
+        return self, S.One
 
     def count_ops(self, symbolic=True):
         #      f()             args
-        return 1   + Basic.Add(*[t.count_ops(symbolic) for t in self])
+        return 1   + Add(*[t.count_ops(symbolic) for t in self.args])
 
     def _eval_oseries(self, order):
         assert self.func.nargs==1,`self.func`
-        arg = self[0]
+        arg = self.args[0]
         x = order.symbols[0]
-        if not Basic.Order(1,x).contains(arg):
+        if not C.Order(1,x).contains(arg):
             return self.func(arg)
         arg0 = arg.limit(x, 0)
-        if not isinstance(arg0, Basic.Zero):
+        if arg0 is not S.Zero:
             e = self.func(arg)
             e1 = e.expand()
             if e==e1:
-                #one example is e = sin(x+1)
+                #for example when e = sin(x+1) or e = sin(cos(x))
                 #let's try the general algorithm
-                term = e.subs(x, Basic.Rational(0))
-                series = Basic.Rational(0)
-                fact = Basic.Rational(1)
+                term = e.subs(x, S.Zero)
+                series = S.Zero
+                fact = S.One
                 i = 0
-                while not order.contains(term):
+                while not order.contains(term) or term == 0:
                     series += term
                     i += 1
-                    fact *= Basic.Rational(i)
+                    fact *= Rational(i)
                     e = e.diff(x)
-                    term = e.subs(x, Basic.Rational(0))*(x**i)/fact
+                    term = e.subs(x, S.Zero)*(x**i)/fact
                 return series
 
                 #print '%s(%s).oseries(%s) is unevaluated' % (self.func,arg,order)
             return e1.oseries(order)
         return self._compute_oseries(arg, order, self.func.taylor_term, self.func)
 
+    def nseries(self, x, x0, n):
+        return self.series(x, x0, n)
+
     def _eval_is_polynomial(self, syms):
-        for arg in self:
+        for arg in self.args:
             if arg.has(*syms):
                 return False
         return True
 
     def _eval_expand_complex(self, *args):
-        func = self.func(*[ a._eval_expand_complex(*args) for a in self ])
-        return Basic.Re()(func) + S.ImaginaryUnit * Basic.Im()(func)
+        func = self.func(*[ a._eval_expand_complex(*args) for a in self.args ])
+        return C.re(func) + S.ImaginaryUnit * C.im(func)
 
     def _eval_rewrite(self, pattern, rule, **hints):
         if hints.get('deep', False):
             args = [ a._eval_rewrite(pattern, rule, **hints) for a in self ]
         else:
-            args = self[:]
+            args = self.args[:]
 
         if pattern is None or isinstance(self.func, pattern):
             if hasattr(self, rule):
@@ -317,12 +311,18 @@ class Function(Basic, ArithMeths, RelMeths):
             else:
                 nargs = self.nargs
             if not (1<=argindex<=nargs):
-                raise TypeError("argument index %r is out of range [1,%s]" % (i,nargs))
-        return Derivative(self,self[argindex-1],evaluate=False)
+                raise TypeError("argument index %r is out of range [1,%s]" % (argindex,nargs))
+        return Derivative(self,self.args[argindex-1],evaluate=False)
+
+    def torepr(self):
+        r = '%s(%r)' % (self.func.__base__.__name__, self.func.__name__)
+        r+= '(%s)' % ', '.join([a.torepr() for a in self.args])
+        return r
 
     def tostr(self, level=0):
         p = self.precedence
-        r = '%s(%s)' % (self.func.__name__, ', '.join([a.tostr() for a in self]))
+        r = '%s(%s)' % (self.func.__name__, ', '.join([a.tostr() for a in
+            self.args]))
         if p <= level:
             return '(%s)' % (r)
         return r
@@ -333,16 +333,16 @@ class Function(Basic, ArithMeths, RelMeths):
 
         #if cls.nargs == 1:
         # common case for functions with 1 argument
-        #if isinstance(arg, Basic.Number):
+        #if arg.is_Number:
         if arg.is_number:
             func_evalf = getattr(arg, cls.__name__)
             return func_evalf()
 
     def _eval_as_leading_term(self, x):
         """General method for the leading term"""
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
@@ -350,17 +350,29 @@ class Function(Basic, ArithMeths, RelMeths):
     @classmethod
     def taylor_term(cls, n, x, *previous_terms):
         """General method for the taylor term.
-        
+
         This method is slow, because it differentiates n-times.  Subclasses can
-        redefine it to make it faster by using the "previous_terms". 
+        redefine it to make it faster by using the "previous_terms".
         """
-        x = Basic.sympify(x)
-        return cls(x).diff(x, n).subs(x, 0) * x**n / Basic.Factorial(n)
+        x = sympify(x)
+        return cls(x).diff(x, n).subs(x, 0) * x**n / C.Factorial(n)
 
 class WildFunction(Function, Atom):
+    """
+    WildFunction() matches any expression but another WildFunction()
+    XXX is this as intended, does it work ?
+    """
 
     nargs = 1
 
+    def __new__(cls, name=None, **assumptions):
+        if name is None:
+            name = 'Wf%s' % (Symbol.dummycount + 1) # XXX refactor dummy counting
+            Symbol.dummycount += 1
+        obj = Function.__new__(cls, name, **assumptions)
+        obj.name = name
+        return obj
+
     def matches(pattern, expr, repl_dict={}, evaluate=False):
         for p,v in repl_dict.items():
             if p==pattern:
@@ -373,6 +385,9 @@ class WildFunction(Function, Atom):
         repl_dict[pattern] = expr
         return repl_dict
 
+    def torepr(self):
+        return '%s(%r)' % (self.__class__.__name__, self.name)
+
     def tostr(self, level=0):
         return self.name + '_'
 
@@ -380,148 +395,80 @@ class WildFunction(Function, Atom):
     def _eval_apply_evalf(cls, arg):
         return
 
-class Lambda(Function):
-    """
-    Lambda(expr, arg1, arg2, ...) -> lambda arg1, arg2,... : expr
-
-    Lambda instance has the same assumptions as its body.
-
-    """
-    precedence = Basic.Lambda_precedence
-    name = None
-    has_derivative = True
-
-    def __new__(cls, expr, *args):
-        expr = Basic.sympify(expr)
-        args = tuple(map(Basic.sympify, args))
-        #if isinstance(expr, Apply):
-        #    if expr[:]==args:
-        #        return expr.func
-        dummy_args = []
-        for a in args:
-            if not isinstance(a, Basic.Symbol):
-                raise TypeError("%s %s-th argument must be Symbol instance (got %r)" \
-                                % (cls.__name__, len(dummy_args)+1,a))
-            d = a.as_dummy()
-            expr = expr.subs(a, d)
-            dummy_args.append(d)
-        obj = Basic.__new__(cls, expr, *dummy_args, **expr._assumptions)
-        return obj
-
-    def _hashable_content(self):
-        return self._args
-
     @property
-    def nargs(self):
-        return len(self._args)-1
-
-    def __getitem__(self, iter):
-        return self._args[1:][iter]
-
-    def __len__(self):
-        return len(self[:])
-
-    @property
-    def body(self):
-        return self._args[0]
-
-    def tostr(self, level=0):
-        precedence = self.precedence
-        r = 'lambda %s: %s' % (', '.join([a.tostr() for a in self]),
-                               self.body.tostr(precedence))
-        if precedence <= level:
-            return '(%s)' % r
-        return r
-
-    def torepr(self):
-        return '%s(%s)' % (self.__class__.__name__, ', '.join([a.torepr() for a in self]))
-
-    def as_coeff_terms(self, x=None):
-        c,t = self.body.as_coeff_terms(x)
-        return c, [Lambda(Basic.Mul(*t),*self[:])]
-
-    def _eval_power(b, e):
-        """
-        (lambda x:f(x))**e -> (lambda x:f(x)**e)
-        """
-        return Lambda(b.body**e, *b[:])
-
-    def _eval_fpower(b, e):
-        """
-        FPow(lambda x:f(x), 2) -> lambda x:f(f(x)))
-        """
-        if isinstance(e, Basic.Integer) and e.is_positive and e.p < 10 and len(b)==1:
-            r = b.body
-            for i in xrange(e.p-1):
-                r = b(r)
-            return Lambda(r, *b[:])
-
-    def with_dummy_arguments(self, args = None):
-        if args is None:
-            args = tuple([a.as_dummy() for a in self])
-        if len(args) != len(self):
-            raise TypeError("different number of arguments in Lambda functions: %s, %s" % (len(args), len(self)))
-        expr = self.body
-        for a,na in zip(self, args):
-            expr = expr.subs(a, na)
-        return expr, args
-
-    def _eval_expand_basic(self, *args):
-        return Lambda(self.body._eval_expand_basic(*args), *self[:])
-
-    def diff(self, *symbols):
-        return Lambda(self.body.diff(*symbols), *self[:])
-
-    def fdiff(self, argindex=1):
-        if not (1<=argindex<=len(self)):
-            raise TypeError("%s.fderivative() argindex %r not in the range [1,%s]"\
-                            % (self.__class__, argindex, len(self)))
-        s = self[argindex-1]
-        expr = self.body.diff(s)
-        return Lambda(expr, *self[:])
-
-    _eval_subs = Basic._seq_subs
-
-    def canonize(cls, *args):
-        n = cls.nargs
-        if n!=len(args):
-            raise TypeError('%s takes exactly %s arguments (got %s)'\
-                            % (cls, n, len(args)))
-        expr = cls.body
-        for da,a in zip(cls, args):
-            expr = expr.subs(da,a)
-        return expr
+    def is_number(self):
+        return False
 
 class Derivative(Basic, ArithMeths, RelMeths):
     """
     Carries out differentation of the given expression with respect to symbols.
 
-    expr must define ._eval_derivative(symbol) method that returns differentation result or None.
+    expr must define ._eval_derivative(symbol) method that returns
+    the differentation result or None.
+
+    Examples:
 
     Derivative(Derivative(expr, x), y) -> Derivative(expr, x, y)
+    Derivative(expr, x, 3)  -> Derivative(expr, x, x, x)
     """
 
     precedence = Basic.Apply_precedence
 
-    def __new__(cls, expr, *symbols, **assumptions):
-        expr = Basic.sympify(expr)
-        if not symbols: return expr
-        symbols = map(Basic.sympify, symbols)
+    @staticmethod
+    def _symbolgen(*symbols):
+        """
+        Generator of all symbols in the argument of the Derivative.
 
-        if not assumptions.get("evaluate", True):
+        Example:
+        >> ._symbolgen(x, 3, y)
+        (x, x, x, y)
+        >> ._symbolgen(x, 10**6)
+        (x, x, x, x, x, x, x, ...)
+
+        The second example shows why we don't return a list, but a generator,
+        so that the code that calls _symbolgen can return earlier for special
+        cases, like x.diff(x, 10**6).
+        """
+        last_s = sympify(symbols[len(symbols)-1])
+        for i in xrange(len(symbols)):
+            s = sympify(symbols[i])
+            next_s = None
+            if s != last_s:
+                next_s = sympify(symbols[i+1])
+
+            if isinstance(s, Integer):
+                continue
+            elif isinstance(s, Symbol):
+                # handle cases like (x, 3)
+                if isinstance(next_s, Integer):
+                    # yield (x, x, x)
+                    for copy_s in repeat(s,int(next_s)):
+                        yield copy_s
+                else:
+                    yield s
+            elif isinstance(s, Function):
+                yield s
+            else:
+                 raise TypeError("Derivative argument must be Symbol|Integer|Function instance (got %s)" % (s.__class__.__name__))
+
+    def __new__(cls, expr, *symbols, **assumptions):
+        expr = sympify(expr)
+        if not symbols: return expr
+        symbols = Derivative._symbolgen(*symbols)
+        if not assumptions.get("evaluate", False):
             obj = Basic.__new__(cls, expr, *symbols)
             return obj
-
-        for s in symbols:
-            assert isinstance(s, Basic.Symbol),`s`
-            if not expr.has(s):
-                return Basic.Zero()
-
         unevaluated_symbols = []
         for s in symbols:
+            s = sympify(s)
+            assert isinstance(s, Symbol),`s`
+            if not expr.has(s):
+                return S.Zero
             obj = expr._eval_derivative(s)
             if obj is None:
                 unevaluated_symbols.append(s)
+            elif obj is S.Zero:
+                return expr
             else:
                 expr = obj
 
@@ -529,19 +476,6 @@ class Derivative(Basic, ArithMeths, RelMeths):
             return expr
         return Basic.__new__(cls, expr, *unevaluated_symbols)
 
-    def xas_apply(self):
-        # Derivative(f(x),x) -> Apply(Lambda(f(_x),_x), x)
-        symbols = []
-        indices = []
-        for s in self.symbols:
-            if s not in symbols:
-                symbols.append(s)
-                indices.append(len(symbols))
-            else:
-                indices.append(symbols.index(s)+1)
-        stop
-        return Apply(FApply(FDerivative(*indices), Lambda(self.expr, *symbols)), *symbols)
-
     def _eval_derivative(self, s):
         #print
         #print self
@@ -555,7 +489,7 @@ class Derivative(Basic, ArithMeths, RelMeths):
         return Derivative(self.expr, *((s,)+self.symbols), **{'evaluate': False})
 
     def doit(self):
-        return Derivative(self.expr, *self.symbols)
+        return Derivative(self.expr, *self.symbols,**{'evaluate': True})
 
     @property
     def expr(self):
@@ -566,13 +500,13 @@ class Derivative(Basic, ArithMeths, RelMeths):
         return self._args[1:]
 
     def tostr(self, level=0):
-        r = 'D' + `tuple(self)`
+        r = 'D' + `tuple(self.args)`
         if self.precedence <= level:
             r = '(%s)' % (r)
         return r
 
     def _eval_subs(self, old, new):
-        return Derivative(self[0].subs(old, new), *self[1:])
+        return Derivative(self.args[0].subs(old, new), *self.args[1:], **{'evaluate': True})
 
     def matches(pattern, expr, repl_dict={}, evaluate=False):
         # this method needs a cleanup.
@@ -600,7 +534,129 @@ class Derivative(Basic, ArithMeths, RelMeths):
         repl_dict[pattern] = expr
         return repl_dict
 
+class Lambda(Function):
+    """
+    Lambda(x, expr) represents a lambda function similar to Python's
+    'lambda x: expr'. A function of several variables is written as
+    Lambda((x, y, ...), expr).
 
+    A simple example:
+        >>> from sympy import Symbol
+        >>> x = Symbol('x')
+        >>> f = Lambda(x, x**2)
+        >>> f(4)
+        16
+
+    For multivariate functions, use:
+        >>> x = Symbol('x')
+        >>> y = Symbol('y')
+        >>> z = Symbol('z')
+        >>> t = Symbol('t')
+        >>> f2 = Lambda(x,y,z,t,x+y**z+t**z)
+        >>> f2(1,2,3,4)
+        73
+
+    Multivariate functions can be curries for partial applications:
+        >>> sum2numbers = Lambda(x,y,x+y)
+        >>> sum2numbers(1,2)
+        3
+        >>> plus1 = sum2numbers(1)
+        >>> plus1(3)
+        4
+
+    A handy shortcut for lots of arguments:
+        >>> from sympy import *
+        >>> var('x y z')
+        (x, y, z)
+        >>> p = x, y, z
+        >>> f = Lambda(p, x + y*z)
+        >>> f(*p)
+        x + y*z
+
+    """
+
+    # a minimum of 2 arguments (parameter, expression) are needed
+    nargs = 2
+    def __new__(cls,*args):
+       # nargs = len(args)
+        assert len(args) >= 2,"Must have at least one parameter and an expression"
+        if len(args) == 2 and isinstance(args[0], (list, tuple)):
+            args = tuple(args[0])+(args[1],)
+        obj = Function.__new__(cls,*args)
+        obj.nargs = len(args)
+        return obj
+
+    @classmethod
+    def canonize(cls,*args):
+        obj = Basic.__new__(cls, *args)
+        #use dummy variables internally, just to be sure
+        nargs = len(args)
+
+        expression = args[nargs-1]
+        funargs = [Symbol(str(arg),dummy=True) for arg in args[:nargs-1]]
+        #probably could use something like foldl here
+        for arg,funarg in zip(args[:nargs-1],funargs):
+            expression = expression.subs(arg,funarg)
+        funargs.append(expression)
+        obj._args = tuple(funargs)
+
+        return obj
+
+    def apply(self, *args):
+        """Applies the Lambda function "self" to the arguments given.
+        This supports partial application.
+
+        Example:
+            >>> from sympy import Symbol
+            >>> x = Symbol('x')
+            >>> y = Symbol('y')
+            >>> f = Lambda(x, x**2)
+            >>> f.apply(4)
+            16
+            >>> sum2numbers = Lambda(x,y,x+y)
+            >>> sum2numbers(1,2)
+            3
+            >>> plus1 = sum2numbers(1)
+            >>> plus1(3)
+            4
+
+        """
+
+        nparams = self.nargs - 1
+        assert nparams >= len(args),"Cannot call function with more parameters than function variables: %s (%d variables) called with %d arguments" % (str(self),nparams,len(args))
+
+
+        #replace arguments
+        expression = self.args[self.nargs-1]
+        for arg,funarg in zip(args,self.args[:nparams]):
+            expression = expression.subs(funarg,arg)
+
+        #curry the rest
+        if nparams != len(args):
+            unused_args = list(self.args[len(args):nparams])
+            unused_args.append(expression)
+            return Lambda(*tuple(unused_args))
+        return expression
+
+    def __call__(self, *args):
+        return self.apply(*args)
+
+    def __eq__(self, other):
+        if isinstance(other, Lambda):
+            if not len(self.args) == len(other.args):
+                return False
+
+            selfexpr = self.args[self.nargs-1]
+            otherexpr = other.args[other.nargs-1]
+            for selfarg,otherarg in zip(self.args[:self.nargs-1],other.args[:other.nargs-1]):
+                otherexpr = otherexpr.subs(otherarg,selfarg)
+            if selfexpr == otherexpr:
+                return True
+           # if self.args[1] == other.args[1].subs(other.args[0], self.args[0]):
+           #     return True
+        return False
+
+@vectorize(0,1,2)
 def diff(f, x, times = 1, evaluate=True):
     """Differentiate f with respect to x
 
@@ -609,10 +665,46 @@ def diff(f, x, times = 1, evaluate=True):
 
     see http://documents.wolfram.com/v5/Built-inFunctions/AlgebraicComputation/Calculus/D.html
     """
-    f = Basic.sympify(f)
-    if evaluate == True:
-        for i in range(0,times):
-            f = f.diff(x)
-        return f
-    else:
-        return Derivative(f, x, evaluate=evaluate)
+
+    return Derivative(f,x,times, **{'evaluate':evaluate})
+
+@vectorize(0)
+def expand(e, **hints):
+    """
+    Expand an expression using hints.
+
+    This is just a wrapper around Basic.expand(), see it's docstring of for a
+    thourough docstring for this function. In isympy you can just type
+    Basic.expand? and enter.
+    """
+    return sympify(e).expand(**hints)
+
+
+# /cyclic/
+import basic as _
+_.Derivative    = Derivative
+_.FunctionClass = FunctionClass
+del _
+
+import add as _
+_.FunctionClass = FunctionClass
+del _
+
+import mul as _
+_.FunctionClass = FunctionClass
+_.WildFunction  = WildFunction
+del _
+
+import operations as _
+_.Lambda        = Lambda
+_.WildFunction  = WildFunction
+del _
+
+import symbol as _
+_.Function      = Function
+_.WildFunction  = WildFunction
+del _
+
+import numbers as _
+_.FunctionClass = FunctionClass
+del _
