@@ -1,18 +1,41 @@
 import math
 import decimal
 import decimal_math
-from basic import Basic, Atom, Singleton, S, Memoizer, MemoizerArg
+from basic import Basic, Atom, Singleton, S, C, Memoizer, MemoizerArg
+from sympify import _sympify, SympifyError, _sympifyit
 from methods import NoRelMeths, RelMeths, ArithMeths
 from power import integer_nthroot
 
-@Memoizer((int, long), (int, long))
+# from mul import Mul   /cyclic/
+# from power import Pow /cyclic/
+# from function import FunctionClass    /cyclic/
+
+# (a,b) -> gcd(a,b)
+_gcdcache = {}
+
+# TODO caching with decorator, but not to degrade performance
 def gcd(a, b):
     """
     Returns the Greatest Common Divisor, implementing Euclid's algorithm.
     """
-    while a:
-        a, b = b%a, a
-    return b
+    try:
+        return _gcdcache[(a,b)]
+    except KeyError:
+        key = (a,b)
+        while a:
+            a, b = b%a, a
+
+        _gcdcache[key] = b
+        return b
+
+igcd = gcd # TODO: rename gcd -> igcd in all modules
+
+def ilcm(a, b):
+    """Computes integer least common multiple of two numbers. """
+    if a == 0 and b == 0:
+        return 0
+    else:
+        return a * b / igcd(a, b)
 
 @Memoizer((int, long), return_value_converter = lambda d: d.copy())
 def factor_trial_division(n):
@@ -71,6 +94,10 @@ class Number(Atom, RelMeths, ArithMeths):
     is_bounded = True
     is_finite = True
 
+    __slots__ = []
+
+    is_Number = True
+
     def __new__(cls, *obj):
         if len(obj)==1: obj=obj[0]
         if isinstance(obj, (int, long)):
@@ -101,7 +128,7 @@ class Number(Atom, RelMeths, ArithMeths):
 
     def _eval_order(self, *symbols):
         # Order(5, x, y) -> Order(1,x,y)
-        return Basic.Order(S.One, *symbols)
+        return C.Order(S.One, *symbols)
 
     def sqrt(self): return Real(decimal_math.sqrt(self._as_decimal()))
     def exp(self): return Real(decimal_math.exp(self._as_decimal()))
@@ -135,13 +162,13 @@ class Number(Atom, RelMeths, ArithMeths):
         raise NotImplementedError,'%s needs .__le__() method' % (self.__class__.__name__)
 
     def __gt__(self, other):
-        return Basic.sympify(other).__lt__(self)
+        return _sympify(other).__lt__(self)
     def __ge__(self, other):
-        return Basic.sympify(other).__le__(self)
+        return _sympify(other).__le__(self)
 
     def as_coeff_terms(self, x=None):
         # a -> c * t
-        return self, []
+        return self, tuple()
 
 decimal_to_Number_cls = {
     decimal.Decimal('0').as_tuple():'Zero',
@@ -178,6 +205,10 @@ class Real(Number):
     is_irrational = False
     is_integer = False
 
+    __slots__ = ['num']
+
+    is_Real = True
+
     @Memoizer(type, MemoizerArg((str, int, long, float, decimal.Decimal), convert_to_Decimal))
     def __new__(cls, num):
         if isinstance(num, (int, long)):
@@ -185,7 +216,7 @@ class Real(Number):
 
         singleton_cls_name = decimal_to_Number_cls.get(num.as_tuple(), None)
         if singleton_cls_name is not None:
-            return getattr(Basic, singleton_cls_name)()
+            return getattr(C, singleton_cls_name)()
         obj = Basic.__new__(cls)
         obj.num = num
         return obj
@@ -247,14 +278,20 @@ class Real(Number):
         return Real(-self.num)
 
     def __mul__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return NotImplemented
         if isinstance(other, Number):
             return Real(self.num * other._as_decimal())
         return Number.__mul__(self, other)
 
     def __add__(self, other):
-        other = Basic.sympify(other)
-        if isinstance(other, NaN) or isinstance(self, NaN):
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return NotImplemented
+        if (other is S.NaN) or (self is NaN):
             return S.NaN
         if isinstance(other, Number):
             return Real(self.num + other._as_decimal())
@@ -293,25 +330,38 @@ class Real(Number):
         return float(self.num)
 
     def __eq__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy != other  -->  not ==
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return False
             return other.__eq__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
             return bool(self._as_decimal()==other._as_decimal())
-        return RelMeths.__eq__(self, other)
+
+        return False    # Real != non-Number
+
     def __ne__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return True     # sympy != other
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return True
             return other.__ne__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
             return bool(self._as_decimal()!=other._as_decimal())
-        return RelMeths.__ne__(self, other)
+
+        return True     # Real != non-Number
+
     def __lt__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other
         if isinstance(other, NumberSymbol):
             return other.__ge__(self)
         if other.is_comparable: other = other.evalf()
@@ -319,7 +369,10 @@ class Real(Number):
             return bool(self._as_decimal() < other._as_decimal())
         return RelMeths.__lt__(self, other)
     def __le__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other  -->  ! <=
         if isinstance(other, NumberSymbol):
             return other.__gt__(self)
         if other.is_comparable: other = other.evalf()
@@ -328,7 +381,7 @@ class Real(Number):
         return RelMeths.__le__(self, other)
 
     def epsilon_eq(self, other, epsilon="10e-16"):
-        return abs(self - other) < Basic.Real(epsilon)
+        return abs(self - other) < Real(epsilon)
 
 def _parse_rational(s):
     """Parse rational number from string representation"""
@@ -382,11 +435,28 @@ class Rational(Number):
         1193/990
         >>> float(Rational(1193,990))
         1.2050505050505051
+
+
+    Low-level
+    ---------
+
+    Access nominator and denominator as .p and .q:
+        >>> r = Rational(3,4)
+        >>> r
+        3/4
+        >>> r.p
+        3
+        >>> r.q
+        4
     """
     is_real = True
     is_integer = False
     is_rational = True
 
+    __slots__ = ['p', 'q']
+
+    is_Rational = True
+
     @Memoizer(type, (int, long, str), MemoizerArg((int, long, type(None)), name="q"))
     def __new__(cls, p, q = None):
         if q is None:
@@ -410,7 +480,7 @@ class Rational(Number):
         obj = Basic.__new__(cls)
         obj.p = p
         obj.q = q
-        #obj._args = [p, q]
+        #obj._args = (p, q)
         return obj
 
     def _hashable_content(self):
@@ -438,9 +508,9 @@ class Rational(Number):
 
     def __neg__(self): return Rational(-self.p, self.q)
 
+    @_sympifyit('other', NotImplemented)
     def __mul__(self, other):
-        other = Basic.sympify(other)
-        if isinstance(other, NaN) or isinstance(self, NaN):
+        if (other is S.NaN) or (self is S.NaN):
             return S.NaN
         if isinstance(other, Real):
             return Real(self._as_decimal() * other.num)
@@ -448,9 +518,10 @@ class Rational(Number):
             return Rational(self.p * other.p, self.q * other.q)
         return Number.__mul__(self, other)
 
+    # TODO reorder
+    @_sympifyit('other', NotImplemented)
     def __add__(self, other):
-        other = Basic.sympify(other)
-        if isinstance(other, NaN) or isinstance(self, NaN):
+        if (other is S.NaN) or (self is S.NaN):
             return S.NaN
         if isinstance(other, Real):
             return Real(self._as_decimal() + other.num)
@@ -468,16 +539,16 @@ class Rational(Number):
 
     def _eval_power(b, e):
         if isinstance(e, Number):
-            if isinstance(e, NaN): return S.NaN
+            if (e is S.NaN): return S.NaN
             if isinstance(e, Real):
                 return Real(decimal_math.pow(b._as_decimal(), e.num))
             if e.is_negative:
                 # (3/4)**-2 -> (4/3)**2
                 ne = -e
-                if isinstance(ne, One):
+                if (ne is S.One):
                     return Rational(b.q, b.p)
                 return Rational(b.q, b.p) ** ne
-            if isinstance(e, Infinity):
+            if (e is S.Infinity):
                 if b.p > b.q:
                     # (3/2)**oo -> oo
                     return S.Infinity
@@ -493,34 +564,13 @@ class Rational(Number):
                     # (4/3)**(5/6) -> 4**(5/6) * 3**(-5/6)
                     return Integer(b.p) ** e * Integer(b.q) ** (-e)
                 if b >= 0:
-                    x, xexact = integer_nthroot(b.p, e.q)
-                    y, yexact = integer_nthroot(b.q, e.q)
-                    if xexact and yexact:
-                        res = Rational(x ** abs(e.p), y ** abs(e.p))
-                        if e >= 0:
-                            return res
-                        else:
-                            return 1/res
-                    # Now check also devisors of the exponents denominator
-                    # TODO: Check if this slows down to much.
-                    for i in xrange(2, e.q/2 + 1):
-                        if e.q % i == 0:
-                            x, xexact = integer_nthroot(b.p, i)
-                            y, yexact = integer_nthroot(b.q, i)
-                            if xexact and yexact:
-                                return Rational(x, y)**Rational(e.p, e.q/i)
-                    else:
-                        # Try to get some part of the base out, if exp > 1
-                        if e.p > e.q:
-                            i = e.p / e.q
-                            r = e.p % e.q
-                            return b**i * b**Rational(r, e.q)
+                    return Integer(b.q)**Rational(e.p * (e.q-1), e.q) / ( Integer(b.q) ** Integer(e.p))
                 else:
                     return (-1)**e * (-b)**e
 
         c,t = b.as_coeff_terms()
-        if e.is_even and isinstance(c, Basic.Number) and c < 0:
-            return (-c * Basic.Mul(*t)) ** e
+        if e.is_even and isinstance(c, Number) and c < 0:
+            return (-c * Mul(*t)) ** e
 
         return
 
@@ -534,11 +584,13 @@ class Rational(Number):
         return int(self.p//self.q)
 
     def __eq__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy != other  -->  not ==
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return False
             return other.__eq__(self)
-        from sympy.core.function import FunctionClass
         if isinstance(self, Number) and isinstance(other, FunctionClass):
             return False
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
@@ -546,9 +598,14 @@ class Rational(Number):
             if isinstance(other, Real):
                 return bool(self._as_decimal()==other._as_decimal())
             return bool(self.p==other.p and self.q==other.q)
-        return RelMeths.__eq__(self, other)
+
+        return False    # Rational != non-Number
+
     def __ne__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return True     # sympy != other
         if isinstance(other, NumberSymbol):
             if other.is_irrational: return True
             return other.__ne__(self)
@@ -557,9 +614,14 @@ class Rational(Number):
             if isinstance(other, Real):
                 return bool(self._as_decimal()!=other._as_decimal())
             return bool(self.p!=other.p or self.q!=other.q)
-        return RelMeths.__ne__(self, other)
+
+        return True     # Rational != non-Number
+
     def __lt__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other  --> not <
         if isinstance(other, NumberSymbol):
             return other.__ge__(self)
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
@@ -569,7 +631,10 @@ class Rational(Number):
             return bool(self.p * other.q < self.q * other.p)
         return RelMeths.__lt__(self, other)
     def __le__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other  -->  not <=
         if isinstance(other, NumberSymbol):
             return other.__gt__(self)
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
@@ -606,21 +671,117 @@ class Rational(Number):
         #return sage.Integer(self[0])/sage.Integer(self[1])
         return sage.Integer(self.p)/sage.Integer(self.q)
 
+# int -> Integer
+_intcache = {}
+
 class Integer(Rational):
 
     q = 1
     is_integer = True
 
-    @Memoizer(type, (int, long))
+    is_Integer = True
+
+    __slots__ = ['p']
+
+    # TODO caching with decorator, but not to degrade performance
     def __new__(cls, i):
-        if isinstance(i, Integer):
-            return i
-        if i==0: return S.Zero
-        if i==1: return S.One
-        if i==-1: return S.NegativeOne
-        obj = Basic.__new__(cls)
-        obj.p = i
-        return obj
+        try:
+            return _intcache[i]
+        except KeyError:
+            if not isinstance(i, (int, long)):
+                raise ValueError('invalid argument for Integer: %r' % (i,))
+            obj = Basic.__new__(cls)
+            obj.p = i
+            _intcache[i] = obj
+            return obj
+
+    # Arithmetic operations are here for efficiency
+    def __neg__(self):
+        return Integer(-self.p)
+
+    def __abs__(self):
+        if self.p >= 0:
+            return self
+        else:
+            return Integer(-self.p)
+
+    # TODO make it decorator + bytecodehacks?
+    def __add__(a, b):
+        if type(b) is int:
+            return Integer(a.p + b)
+        elif isinstance(b, Integer):
+            return Integer(a.p + b.p)
+        return Rational.__add__(a, b)   # a,b -not- b,a
+
+    def __radd__(a, b):
+        if type(b) is int:
+            return Integer(b + a.p)
+        elif isinstance(b, Integer):
+            return Integer(b.p + a.p)
+        return Rational.__add__(a, b)
+
+    def __mul__(a, b):
+        if type(b) is int:
+            return Integer(a.p * b)
+        elif isinstance(b, Integer):
+            return Integer(a.p * b.p)
+        return Rational.__mul__(a, b)
+
+    def __rmul__(a, b):
+        if type(b) is int:
+            return Integer(b * a.p)
+        elif isinstance(b, Integer):
+            return Integer(b.p * a.p)
+        return Rational.__mul__(a, b)
+
+    # XXX __pow__ ?
+
+    # XXX do we need to define __cmp__ ?
+#   def __cmp__(a, b):
+
+    def __eq__(a, b):
+        if type(b) is int:
+            return (a.p == b)
+        elif isinstance(b, Integer):
+            return (a.p == b.p)
+        return Rational.__eq__(a, b)
+
+    def __ne__(a, b):
+        if type(b) is int:
+            return (a.p != b)
+        elif isinstance(b, Integer):
+            return (a.p != b.p)
+        return Rational.__ne__(a, b)
+
+    def __gt__(a, b):
+        if type(b) is int:
+            return (a.p >  b)
+        elif isinstance(b, Integer):
+            return (a.p >  b.p)
+        return Rational.__gt__(a, b)
+
+    def __lt__(a, b):
+        if type(b) is int:
+            return (a.p <  b)
+        elif isinstance(b, Integer):
+            return (a.p <  b.p)
+        return Rational.__lt__(a, b)
+
+    def __ge__(a, b):
+        if type(b) is int:
+            return (a.p >= b)
+        elif isinstance(b, Integer):
+            return (a.p >= b.p)
+        return Rational.__ge__(a, b)
+
+    def __le__(a, b):
+        if type(b) is int:
+            return (a.p <= b)
+        elif isinstance(b, Integer):
+            return (a.p <= b.p)
+        return Rational.__le__(a, b)
+
+    ########################################
 
     def _eval_is_odd(self):
         return bool(self.p % 2)
@@ -644,16 +805,16 @@ class Integer(Rational):
 
     def _eval_power(b, e):
         if isinstance(e, Number):
-            if isinstance(e, NaN): return S.NaN
+            if e is S.NaN: return S.NaN
             if isinstance(e, Real):
                 return Real(decimal_math.pow(b._as_decimal(), e.num))
             if e.is_negative:
                 # (3/4)**-2 -> (4/3)**2
                 ne = -e
-                if isinstance(ne, One):
+                if ne is S.One:
                     return Rational(1, b.p)
                 return Rational(1, b.p) ** ne
-            if isinstance(e, Infinity):
+            if e is S.Infinity:
                 if b.p > 1:
                     # (3)**oo -> oo
                     return S.Infinity
@@ -679,25 +840,61 @@ class Integer(Rational):
                             return res
                         else:
                             return 1/res
-                    # Now check also devisors of the exponents denominator
-                    # TODO: Check if this slows down to much.
-                    for i in xrange(2, e.q/2 + 1):
-                        if e.q % i == 0:
-                            x, xexact = integer_nthroot(b.p, i)
-                            if xexact:
-                                return Integer(x)**(e * i)
-                    # Try to get some part of the base out, if exponent > 1
-                    if e.p > e.q:
-                        i = e.p / e.q
-                        r = e.p % e.q
-                        return b**i * b**Rational(r, e.q)
+                    else:
+                        if b > 2**32: #Prevent from factorizing too big integers:
+                            for i in xrange(2, e.q/2 + 1): #OLD CODE
+                                if e.q % i == 0:
+                                    x, xexact = integer_nthroot(b.p, i)
+                                    if xexact:
+                                        return Integer(x)**(e * i)
+                            # Try to get some part of the base out, if exponent > 1
+                            if e.p > e.q:
+                                i = e.p / e.q
+                                r = e.p % e.q
+                                return b**i * b**Rational(r, e.q)
+                            return
 
+
+                        dict = b.factors()
+
+                        out_int = 1
+                        sqr_int = 1
+                        sqr_gcd = 0
+
+                        sqr_dict = {}
+
+                        for prime,exponent in dict.iteritems():
+                            exponent *= e.p
+                            div_e = exponent / e.q
+                            div_m = exponent % e.q
+
+                            if div_e > 0:
+                                out_int *= prime**div_e
+                            if div_m > 0:
+                                sqr_dict[prime] = div_m
+
+                        for p,ex in sqr_dict.iteritems():
+                            if sqr_gcd == 0:
+                                sqr_gcd = ex
+                            else:
+                                sqr_gcd = gcd(sqr_gcd, ex)
+
+                        for k,v in sqr_dict.iteritems():
+                            sqr_int *= k**(v/sqr_gcd)
+
+                        if sqr_int == b.p and out_int == 1:
+                            return None
+
+                        return out_int * Pow(sqr_int , Rational(sqr_gcd, e.q))
                 else:
-                    return (-1)**e * (-b)**e
+                    if e.q == 2:
+                        return S.ImaginaryUnit ** e.p * (-b)**e
+                    else:
+                        return None
 
         c,t = b.as_coeff_terms()
-        if e.is_even and isinstance(c, Basic.Number) and c < 0:
-            return (-c * Basic.Mul(*t)) ** e
+        if e.is_even and isinstance(c, Number) and c < 0:
+            return (-c * Mul(*t)) ** e
 
         return
 
@@ -725,6 +922,16 @@ class Zero(Singleton, Integer):
     is_prime = False
     is_composite = True
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.Zero
+
+    @staticmethod
+    def __neg__():
+        return S.Zero
+
     def _eval_power(b, e):
         if e.is_negative:
             return S.Infinity
@@ -737,9 +944,9 @@ class Zero(Singleton, Integer):
             return b
         coeff, terms = e.as_coeff_terms()
         if coeff.is_negative:
-            return S.Infinity ** Basic.Mul(*terms)
-        if not isinstance(coeff, Basic.One):
-            return b ** Basic.Mul(*terms)
+            return S.Infinity ** Mul(*terms)
+        if coeff is not S.One:
+            return b ** Mul(*terms)
 
     def _eval_order(self, *symbols):
         # Order(0,x) -> 0
@@ -752,6 +959,16 @@ class One(Singleton, Integer):
 
     is_prime = True
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.One
+
+    @staticmethod
+    def __neg__():
+        return S.NegativeOne
+
     def _eval_power(b, e):
         return b
 
@@ -763,6 +980,16 @@ class NegativeOne(Singleton, Integer):
     p = -1
     q = 1
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.One
+
+    @staticmethod
+    def __neg__():
+        return S.One
+
     def _eval_power(b, e):
         if e.is_odd: return S.NegativeOne
         if e.is_even: return S.One
@@ -772,11 +999,11 @@ class NegativeOne(Singleton, Integer):
                 s = decimal_math.sin(a)
                 c = decimal_math.cos(a)
                 return Real(s) + Real(c) * S.ImaginaryUnit
-            if isinstance(e, NaN):
+            if e is S.NaN:
                 return S.NaN
-            if isinstance(e, (Infinity, NegativeInfinity)):
+            if e is S.Infinity  or  e is S.NegativeInfinity:
                 return S.NaN
-            if isinstance(e, Half):
+            if e is S.Half:
                 return S.ImaginaryUnit
             if isinstance(e, Rational):
                 if e.q == 2:
@@ -792,17 +1019,34 @@ class Half(Singleton, Rational):
     p = 1
     q = 2
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.Half
+
+
 class Infinity(Singleton, Rational):
 
     p = 1
     q = 0
 
+    __slots__ = []
+
     is_commutative = True
     is_positive = True
     is_bounded = False
     is_finite = None
     is_odd = None
 
+    @staticmethod
+    def __abs__():
+        return S.Infinity
+
+    @staticmethod
+    def __neg__():
+        return S.NegativeInfinity
+
     def tostr(self, level=0):
         return 'oo'
 
@@ -818,7 +1062,7 @@ class Infinity(Singleton, Rational):
         if e.is_negative:
             return S.Zero
         if isinstance(e, Number):
-            if isinstance(e, NaN):
+            if e is S.NaN:
                 return S.NaN
         d = e.evalf()
         if isinstance(d, Number):
@@ -833,6 +1077,8 @@ class NegativeInfinity(Singleton, Rational):
     p = -1
     q = 0
 
+    __slots__ = []
+
     is_commutative = True
     is_real = True
     is_positive = False
@@ -841,6 +1087,14 @@ class NegativeInfinity(Singleton, Rational):
 
     precedence = 40 # same as Add
 
+    @staticmethod
+    def __abs__():
+        return S.Infinity
+
+    @staticmethod
+    def __neg__():
+        return S.Infinity
+
     def tostr(self, level=0):
         return '-oo'
 
@@ -856,7 +1110,7 @@ class NegativeInfinity(Singleton, Rational):
 
         """
         if isinstance(e, Number):
-            if isinstance(e, (NaN, Infinity, NegativeInfinity)):
+            if (e is S.NaN)  or  (e is S.Infinity)  or  (e is S.NegativeInfinity):
                 return S.NaN
             if isinstance(e, Integer):
                 if e.is_positive:
@@ -880,6 +1134,8 @@ class NaN(Singleton, Rational):
     is_bounded = None
     #is_unbounded = False
 
+    __slots__ = []
+
     def tostr(self, level=0):
         return 'nan'
 
@@ -887,7 +1143,7 @@ class NaN(Singleton, Rational):
         return decimal.Decimal('NaN')
 
     def _eval_power(b, e):
-        if isinstance(e, Basic.Zero):
+        if e is S.Zero:
             return S.One
         return b
 
@@ -898,15 +1154,25 @@ class ComplexInfinity(Singleton, Atom, NoRelMeths, ArithMeths):
     is_bounded = False
     is_real = None
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.Infinity
+
+    @staticmethod
+    def __neg__():
+        return S.ComplexInfinity
+
     def tostr(self, level=0):
         return 'zoo'
 
     def _eval_power(b, e):
-        if isinstance(e, Basic.ComplexInfinity):
+        if e is S.ComplexInfinity:
             return S.NaN
 
-        if isinstance(e, Basic.Number):
-            if isinstance(e, Basic.Zero):
+        if isinstance(e, Number):
+            if e is S.Zero:
                 return S.NaN
             else:
                 if e.is_positive:
@@ -921,6 +1187,10 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
     is_bounded = True
     is_finite = True
 
+    __slots__ = []
+
+    is_NumberSymbol = True
+
     def approximation(self, number_cls):
         """ Return an interval with number_cls endpoints
         that contains the value of NumberSymbol.
@@ -930,17 +1200,30 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
     def _eval_derivative(self, s):
         return S.Zero
     def __eq__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy != other  -->  not ==
         if self is other: return True
         if isinstance(other, Number) and self.is_irrational: return False
-        return RelMeths.__eq__(self, other)
+
+        return False    # NumberSymbol != non-(Number|self)
+
     def __ne__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return True     # sympy != other
         if self is other: return False
         if isinstance(other, Number) and self.is_irrational: return True
-        return RelMeths.__ne__(self, other)
+
+        return True     # NumberSymbol != non(Number|self)
+
     def __lt__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other  --> not <
         if self is other: return False
         if isinstance(other, Number):
             approx = self.approximation_interval(other.__class__)
@@ -954,7 +1237,10 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
             return self.evalf()<other
         return RelMeths.__lt__(self, other)
     def __le__(self, other):
-        other = Basic.sympify(other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            return False    # sympy > other  --> not <=
         if self is other: return True
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
@@ -973,6 +1259,12 @@ class Exp1(NumberSymbol):
     is_negative = False # XXX Forces is_negative/is_nonnegative
     is_irrational = True
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.Exp1
+
     def tostr(self, level=0):
         return 'E'
 
@@ -986,7 +1278,7 @@ class Exp1(NumberSymbol):
             pass
 
     def _eval_power(self, exp):
-        return Basic.exp(exp)
+        return C.exp(exp)
 
 class Pi(NumberSymbol):
 
@@ -995,6 +1287,12 @@ class Pi(NumberSymbol):
     is_negative = False
     is_irrational = True
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.Pi
+
     def _eval_evalf(self):
         return Real(decimal_math.pi())
 
@@ -1014,6 +1312,8 @@ class GoldenRatio(NumberSymbol):
     is_negative = False
     is_irrational = True
 
+    __slots__ = []
+
     def _eval_evalf(self):
         return Real(decimal_math.golden_ratio())
 
@@ -1036,6 +1336,8 @@ class EulerGamma(NumberSymbol):
     is_negative = False
     is_irrational = None
 
+    __slots__ = []
+
     def _eval_evalf(self):
         return
 
@@ -1055,6 +1357,8 @@ class Catalan(NumberSymbol):
     is_negative = False
     is_irrational = None
 
+    __slots__ = []
+
     def _eval_evalf(self):
         return
 
@@ -1074,6 +1378,12 @@ class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
     is_bounded = True
     is_finite = True
 
+    __slots__ = []
+
+    @staticmethod
+    def __abs__():
+        return S.One
+
     def tostr(self, level=0):
         return 'I'
 
@@ -1112,6 +1422,41 @@ class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
     def as_base_exp(self):
         return -S.One, S.Half
 
+# /cyclic/
+import basic as _
+_.Number    = Number
+_.Integer   = Integer
+_.Rational  = Rational
+_.Real      = Real
+del _
+
+import add as _
+_.Number    = Number
+del _
+
+import mul as _
+_.Number    = Number
+_.Integer   = Integer
+_.Real      = Real
+del _
+
+import power as _
+_.Number    = Number
+_.Rational  = Rational
+_.Integer   = Integer
+del _
+
+import sympify as _
+_.Integer   = Integer
+_.Real      = Real
+del _
+
+# ----
+
+_intcache[0] = S.Zero
+_intcache[1] = S.One
+_intcache[-1]= S.NegativeOne
+
 Basic.singleton['E'] = Exp1
 Basic.singleton['pi'] = Pi
 Basic.singleton['I'] = ImaginaryUnit
