@@ -1,22 +1,31 @@
 
-from basic import Basic, cache_it, cache_it_immutable
+from basic import Basic, S, C
+from sympify import _sympify
+from cache import cacheit
+
+# from add import Add   /cyclic/
+# from mul import Mul   /cyclic/
+# from function import Lambda, WildFunction /cyclic/
+from symbol import Symbol, Wild
 
 class AssocOp(Basic):
     """ Associative operations, can separate noncommutative and
     commutative parts.
-    
+
     (a op b) op c == a op (b op c) == a op b op c.
-    
+
     Base class for Add and Mul.
     """
 
-    @cache_it_immutable
+    __slots__ = []
+
+    @cacheit
     def __new__(cls, *args, **assumptions):
         if len(args)==0:
             return cls.identity()
         if len(args)==1:
-            return Basic.sympify(args[0])
-        c_part, nc_part, lambda_args, order_symbols = cls.flatten(map(Basic.sympify, args))
+            return _sympify(args[0])
+        c_part, nc_part, lambda_args, order_symbols = cls.flatten(map(_sympify, args))
         if len(c_part) + len(nc_part) <= 1:
             if c_part: obj = c_part[0]
             elif nc_part: obj = nc_part[0]
@@ -25,18 +34,18 @@ class AssocOp(Basic):
             assumptions['commutative'] = not nc_part
             obj = Basic.__new__(cls, *(c_part + nc_part), **assumptions)
         if order_symbols is not None:
-            obj = Basic.Order(obj, *order_symbols)
+            obj = C.Order(obj, *order_symbols)
         if lambda_args is not None:
-            obj = Basic.Lambda(obj, *lambda_args)
+            obj = Lambda(obj, *lambda_args)
         return obj
 
     @classmethod
     def identity(cls):
-        if cls is Basic.Mul: return Basic.One()
-        if cls is Basic.Add: return Basic.Zero()
-        if cls is Basic.Composition:
-            s = Basic.Symbol('x',dummy=True)
-            return Basic.Lambda(s,s)
+        if cls is Mul: return S.One
+        if cls is Add: return S.Zero
+        if cls is C.Composition:
+            s = Symbol('x',dummy=True)
+            return Lambda(s,s)
         raise NotImplementedError,"identity not defined for class %r" % (cls.__name__)
 
     @classmethod
@@ -70,8 +79,8 @@ class AssocOp(Basic):
         # eliminate exact part from pattern: (2+a+w1+w2).matches(expr) -> (w1+w2).matches(expr-a-2)
         wild_part = []
         exact_part = []
-        for p in pattern:
-            if p.atoms(type=(Basic.Wild, Basic.WildFunction)):
+        for p in pattern.args:
+            if p.atoms(type=(Wild, WildFunction)):
                 wild_part.append(p)
             else:
                 exact_part.append(p)
@@ -82,7 +91,7 @@ class AssocOp(Basic):
 
         # now to real work ;)
         if isinstance(expr, pattern.__class__):
-            expr_list = list(expr)
+            expr_list = list(expr.args)
         else:
             expr_list = [expr]
 
@@ -101,11 +110,11 @@ class AssocOp(Basic):
     def _eval_template_is_attr(self, is_attr):
         # return True if all elements have the property
         r = True
-        for t in self:
+        for t in self.args:
             a = getattr(t, is_attr)
             if a is None: return
             if r and not a: r = False
         return r
 
     _eval_evalf = Basic._seq_eval_evalf
-    
+
