@@ -1,7 +1,9 @@
 
-from basic import Basic, Atom, S, cache_it
-from basic_methods import cache_it_nondummy
+from basic import Basic, Atom, S, C, sympify
 from methods import RelMeths, ArithMeths
+from cache import cacheit
+
+# from function import Function, WildFunction   /cyclic/
 
 class Symbol(Atom, RelMeths, ArithMeths):
     """
@@ -18,9 +20,11 @@ class Symbol(Atom, RelMeths, ArithMeths):
     """
 
     is_comparable = False
-    dummycount = 0
 
-    #@cache_it_nondummy
+    __slots__ = ['name']
+
+    is_Symbol = True
+
     def __new__(cls, name, commutative=True, dummy=False,
                 **assumptions):
         """if dummy == True, then this Symbol is totally unique, i.e.::
@@ -34,40 +38,40 @@ class Symbol(Atom, RelMeths, ArithMeths):
         False
 
         """
+
+        # XXX compatibility stuff
+        if dummy==True:
+            return Dummy(name, commutative=commutative, **assumptions)
+        else:
+            return Symbol.__xnew_cached_(cls, name, commutative, **assumptions)
+
+    def __new_stage2__(cls, name, commutative=True, **assumptions):
         obj = Basic.__new__(cls,
                             commutative=commutative,
-                            dummy=dummy,
                             **assumptions)
-        if dummy:
-            Symbol.dummycount += 1
-            obj.dummy_index = Symbol.dummycount
         assert isinstance(name, str),`type(name)`
         obj.name = name
         return obj
 
+    __xnew__       = staticmethod(__new_stage2__)            # never cached (e.g. dummy)
+    __xnew_cached_ = staticmethod(cacheit(__new_stage2__))   # symbols are always cached
+
     def _hashable_content(self):
-        if self.is_dummy:
-            return (self.name, self.dummy_index)
         return (self.name,)
 
     def tostr(self, level=0):
-        if self.is_dummy:
-            return '_' + self.name
         return self.name
 
     def torepr(self):
-        if self.is_dummy:
-            return '%s(%r, dummy=True)' % (self.__class__.__name__, self.name)
         return '%s(%r)' % (self.__class__.__name__, self.name)
 
     def as_dummy(self):
         assumptions = self._assumptions.copy()
-        assumptions['dummy'] = True
-        return self.__class__(self.name, **assumptions)
+        return Dummy(self.name, **assumptions)
 
     def __call__(self, *args):
         assumptions = self._assumptions
-        return Basic.Function(self.name, nargs=len(args))(*args, **assumptions)
+        return Function(self.name, nargs=len(args))(*args, **assumptions)
 
     def _eval_integral(self, s):
         if self==s:
@@ -80,28 +84,85 @@ class Symbol(Atom, RelMeths, ArithMeths):
         return self*(b-a)
 
     def _eval_expand_complex(self, *args):
-        return Basic.re(self) + Basic.im(self)*S.ImaginaryUnit
+        return C.re(self) + C.im(self)*S.ImaginaryUnit
 
     def _sage_(self):
         import sage.all as sage
         return sage.var(self.name)
 
+    @property
+    def is_number(self):
+        return False
+
+class Dummy(Symbol):
+    """Dummy Symbol
+
+       use this through Symbol:
+
+       >>> x1 = Symbol('x', dummy=True)
+       >>> x2 = Symbol('x', dummy=True)
+       >>> bool(x1 == x2)
+       False
+    """
+
+    dummycount = 0
+
+    __slots__ = ['dummy_index']
+
+    def __new__(cls, name, commutative=True, **assumptions):
+        obj = Symbol.__xnew__(cls, name, commutative=commutative, **assumptions)
+
+        Dummy.dummycount += 1
+        obj.dummy_index = Dummy.dummycount
+        return obj
+
+    def _hashable_content(self):
+        return (self.name, self.dummy_index)
+
+    def tostr(self, level=0):
+        return '_' + self.name
+
+
+class Temporary(Dummy):
+    """
+    Indexed dummy symbol.
+    """
+
+    __slots__ = []
+
+    def __new__(cls, **assumptions):
+        obj = Dummy.__new__(cls, 'T%i' % Dummy.dummycount, **assumptions)
+        return obj
+
+
 class Wild(Symbol):
     """
     Wild() matches any expression but another Wild().
     """
 
-    def __new__(cls, name=None, exclude=None, **assumptions):
-        assumptions['dummy'] = True
-        if name is None:
-            name = 'W%s' % (Symbol.dummycount+1)
-        obj = Symbol.__new__(cls, name, **assumptions)
+    __slots__ = ['exclude']
+
+    def __new__(cls, name, exclude=None, **assumptions):
+        if type(exclude) is list:
+            exclude = tuple(exclude)
+
+        return Wild.__xnew__(cls, name, exclude, **assumptions)
+
+    @staticmethod
+    @cacheit
+    def __xnew__(cls, name, exclude, **assumptions):
+        obj = Symbol.__xnew__(cls, name, **assumptions)
+
         if exclude is None:
             obj.exclude = None
         else:
-            obj.exclude = [Basic.sympify(x) for x in exclude]
+            obj.exclude = tuple([sympify(x) for x in exclude])
         return obj
 
+    def _hashable_content(self):
+        return (self.name, self.exclude)
+
+    # TODO add check against another Wild
     def matches(pattern, expr, repl_dict={}, evaluate=False):
         for p,v in repl_dict.items():
             if p==pattern:
@@ -118,26 +179,18 @@ class Wild(Symbol):
         return repl_dict
 
     def __call__(self, *args, **assumptions):
-        return Basic.WildFunction(self.name, nargs=len(args))(*args, **assumptions)
+        return WildFunction(self.name, nargs=len(args))(*args, **assumptions)
 
     def tostr(self, level=0):
         return self.name + '_'
 
-class Temporary(Symbol):
-    """
-    Indexed dummy symbol.
-    """
-    def __new__(cls, **assumptions):
-        assumptions['dummy'] = True
-        name = 'T%s' % (Symbol.dummycount+1)
-        return Symbol.__new__(cls, name, **assumptions)
 
 def symbols(*names, **kwargs):
     """Returns a list of symbols with names taken from 'names'
        argument, which can be a string, then each character
        forms a separate symbol, or a sequence of strings.
 
-       All newly created symbols have assumptions set acordingly
+       All newly created symbols have assumptions set accordingly
        to 'kwargs'. Main intention behind this function is to
        simplify and shorten examples code in doc-strings.
 
@@ -179,6 +232,8 @@ def var(s):
 
         EXAMPLES:
         We define some symbolic variables:
+            >>> var('m')
+            m
             >>> var('n xx yy zz')
             (n, xx, yy, zz)
             >>> n
@@ -195,13 +250,29 @@ def var(s):
         res = []
 
         for t in s:
+            # skip empty strings
+            if not t:
+                continue
             sym = Symbol(t)
             frame.f_globals[t] = sym
             res.append(sym)
 
-        return tuple(res)
+        res = tuple(res)
+        if len(res) == 0:   # var('')
+            res = None
+        elif len(res) == 1: # var('x')
+            res = res[0]
+                            # otherwise var('a b ...')
+        return res
 
     finally:
         # we should explicitly break cyclic dependencies as stated in inspect
         # doc
         del frame
+
+# /cyclic/
+import basic as _
+_.Symbol    = Symbol
+_.Wild      = Wild
+_.Temporary = Temporary
+del _
