@@ -1,10 +1,13 @@
-from sympy import Symbol, sin, cos, exp, O, sqrt, Rational, re
+from sympy import Symbol, sin, cos, exp, O, sqrt, Rational, Real, re, pi, \
+        sympify, sqrt
 from sympy.utilities.pytest import XFAIL
 
+x = Symbol("x")
+y = Symbol('y')
+
 def test_bug1():
-    x=Symbol("x")
     assert re(x) != x
-    x.series(x,1)
+    x.series(x,0,1)
     assert re(x) != x
 
 a = Symbol("a")
@@ -20,7 +23,7 @@ def test_Symbol():
     assert a*b*b+c==c+a*b**2
     assert a*b*b-c==-c+a*b**2
 
-def test_arit():
+def test_arit0():
     p = Rational(5)
     e=a*b
     assert e == a*b
@@ -146,7 +149,7 @@ def testpow():
 
 @XFAIL
 def test_pow2():
-    # XXX These fail - they are maybe discutable, 
+    # XXX These fail - they are maybe discutable,
     # let's see SAGE and similar.
     assert ((-x)**2)**Rational(1,3) == ((-x)**Rational(1,3))**2
     assert (-x)**Rational(2,3) == x**Rational(2,3)
@@ -182,7 +185,7 @@ def test_expand():
     assert e.expand() == 5*a+5*b+5*c+2*a*c+b*c+a*b+a**2+c**2
     x=Symbol("x")
     s=exp(x*x)-1
-    e=s.series(x,5)/x**2
+    e=s.series(x,0,5)/x**2
     assert e.expand() ==  1+x**2/2+O(x**3)
 
     # Check that this isn't too slow
@@ -204,6 +207,11 @@ def test_power_expand():
     p = (1+2*(1+a))**2
     assert p.expand() == 9 + 4*(a**2) + 12*a
 
+def test_real_mul():
+    Real(0) * pi * x == Real(0)
+    Real(1) * pi * x == pi * x
+    len((Real(2) * pi * x).args) == 3
+
 def test_ncmul():
     A = Symbol("A", commutative=False)
     B = Symbol("B", commutative=False)
@@ -757,6 +765,19 @@ def test_Pow_is_integer():
     assert (k**(n*m)).is_integer == True
     assert (k**(-n*m)).is_integer == None
 
+
+def test_Pow_is_real():
+    x = Symbol('x', real=True)
+    y = Symbol('y', real=True, positive=True)
+
+    assert (x**2).is_real   == True
+    assert (x**3).is_real   == True
+    assert (x**x).is_real   == None
+    assert (y**x).is_real   == True
+
+    assert (x**Rational(1,3)).is_real   == None
+    assert (y**Rational(1,3)).is_real   == True
+
 @XFAIL
 def test_Pow_is_bounded():
     x = Symbol('x', real=True)
@@ -805,6 +826,8 @@ def test_Pow_is_negative_positive():
     n = Symbol('n', even=True)
     m = Symbol('m', odd=True)
 
+    z = Symbol('z')
+
     assert (2**x).is_positive == True
     assert ((-2)**x).is_positive == None
     assert ((-2)**n).is_positive == True
@@ -831,6 +854,9 @@ def test_Pow_is_negative_positive():
     assert ((-k)**n).is_negative == False
     assert ((-k)**m).is_negative == True
 
+    assert (2**z).is_positive   == None
+    assert (2**z).is_negative   == None
+
 def test_Pow_is_nonpositive_nonnegative():
     x = Symbol('x', real=True)
 
@@ -863,3 +889,55 @@ def test_Pow_is_nonpositive_nonnegative():
     assert ((-k)**x).is_nonpositive == None
     assert ((-k)**n).is_nonpositive == None
     assert ((-k)**m).is_nonpositive == True
+
+def test_Add_is_comparable():
+    assert (x+y).is_comparable  == False
+    assert (x+1).is_comparable  == False
+    assert (Rational(1,3) - sqrt(8)).is_comparable == True
+
+def test_Mul_is_comparable():
+    assert (x*y).is_comparable  == False
+    assert (x*2).is_comparable  == False
+    assert (sqrt(2)*Rational(1,3)).is_comparable == True
+
+
+def test_Pow_is_comparable():
+    assert (x**y).is_comparable == False
+    assert (x**2).is_comparable == False
+    assert (Rational(1,3)**Rational(1,2)).is_comparable == True
+
+
+def test_Add_is_positive_2():
+    e = Rational(1,3) - sqrt(8)
+    assert e.is_positive    == False
+    assert e.is_negative    == True
+
+    e = pi - 1
+    assert e.is_positive    == True
+    assert e.is_negative    == False
+
+
+def test_issue432():
+    class MightyNumeric(tuple):
+        def __rdiv__(self, other):
+            return "something"
+    assert sympify(1)/MightyNumeric((1,2)) == "something"
+
+def test_issue432b():
+    class Foo:
+        def __init__(self):
+            self.field = 1.0
+        def __mul__(self, other):
+            self.field = self.field * other
+        def __rmul__(self, other):
+            self.field = other * self.field
+    f = Foo()
+    x = Symbol("x")
+    assert f*x == x*f
+
+def test_bug3():
+    a = Symbol("a")
+    b = Symbol("b", positive=True)
+    e = 2*a + b
+    f = b + 2*a
+    assert e == f
