@@ -1,6 +1,9 @@
 
-from sympy.core.basic import Basic, S, cache_it, cache_it_immutable
+from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Function, Lambda
+from sympy.core.cache import cacheit
+
+from sympy.functions.elementary.miscellaneous import sqrt
 
 ###############################################################################
 ########################### HYPERBOLIC FUNCTIONS ##############################
@@ -12,7 +15,7 @@ class sinh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return cosh(self[0])
+            return cosh(self.args[0])
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -25,16 +28,16 @@ class sinh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeInfinity
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
             elif arg.is_negative:
                 return -cls(-arg)
@@ -42,34 +45,35 @@ class sinh(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.sin(i_coeff)
+                return S.ImaginaryUnit * C.sin(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
                 return p * x**2 / (n*(n-1))
             else:
-                return x**(n) / Basic.Factorial(n)
+                return x**(n) / C.Factorial(n)
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        return sinh(re)*Basic.cos(im) + cosh(re)*Basic.sin(im)*S.ImaginaryUnit
+        re, im = self.args[0].as_real_imag()
+        return sinh(re)*C.cos(im) + cosh(re)*C.sin(im)*S.ImaginaryUnit
 
     def _eval_rewrite_as_exp(self, arg):
         return (S.Exp(arg) - S.Exp(-arg)) / 2
@@ -86,17 +90,18 @@ class sinh(Function):
         return 2*coth_half/(coth_half**2 - 1)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self[0].is_real
+        return self.args[0].is_real
+
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
         if arg.is_imaginary:
             return True
 
@@ -106,7 +111,7 @@ class cosh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return sinh(self[0])
+            return sinh(self.args[0])
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -119,16 +124,16 @@ class cosh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.Infinity
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.One
             elif arg.is_negative:
                 return cls(-arg)
@@ -136,34 +141,35 @@ class cosh(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return Basic.cos(i_coeff)
+                return C.cos(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 1:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
                 return p * x**2 / (n*(n-1))
             else:
-                return x**(n)/Basic.Factorial(n)
+                return x**(n)/C.Factorial(n)
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        return cosh(re)*Basic.cos(im) + sinh(re)*Basic.sin(im)*S.ImaginaryUnit
+        re, im = self.args[0].as_real_imag()
+        return cosh(re)*C.cos(im) + sinh(re)*C.sin(im)*S.ImaginaryUnit
 
     def _eval_rewrite_as_exp(self, arg):
         return (S.Exp(arg) + S.Exp(-arg)) / 2
@@ -180,17 +186,18 @@ class cosh(Function):
         return (coth_half+1)/(coth_half-1)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self[0].is_real
+        return self.args[0].is_real
+
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
         if arg.is_imaginary:
             return True
 
@@ -200,7 +207,7 @@ class tanh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return S.One - tanh(self[0])**2
+            return S.One - tanh(self.args[0])**2
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -213,16 +220,16 @@ class tanh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.One
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeOne
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
             elif arg.is_negative:
                 return -cls(-arg)
@@ -230,37 +237,38 @@ class tanh(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.tan(i_coeff)
+                return S.ImaginaryUnit * C.tan(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             a = 2**(n+1)
 
-            B = S.Bernoulli(n+1)
-            F = Basic.Factorial(n+1)
+            B = C.bernoulli(n+1)
+            F = C.Factorial(n+1)
 
             return a*(a-1) * B/F * x**n
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        denom = sinh(re)**2 + Basic.cos(im)**2
+        re, im = self.args[0].as_real_imag()
+        denom = sinh(re)**2 + C.cos(im)**2
         return (sinh(re)*cosh(re) + \
-            S.ImaginaryUnit*Basic.sin(im)*Basic.cos(im))/denom
+            S.ImaginaryUnit*C.sin(im)*C.cos(im))/denom
 
     def _eval_rewrite_as_exp(self, arg):
         neg_exp, pos_exp = S.Exp(-arg), S.Exp(arg)
@@ -276,17 +284,18 @@ class tanh(Function):
         return 1/coth(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self[0].is_real
+        return self.args[0].is_real
+
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
         if arg.is_real:
             return True
 
@@ -296,7 +305,7 @@ class coth(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/sinh(self[0])**2
+            return 1/sinh(self.args[0])**2
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -309,16 +318,16 @@ class coth(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.One
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeOne
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
             elif arg.is_negative:
                 return -cls(-arg)
@@ -326,37 +335,38 @@ class coth(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return -S.ImaginaryUnit * Basic.cot(i_coeff)
+                return -S.ImaginaryUnit * C.cot(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
-            return 1 / Basic.sympify(x)
+            return 1 / sympify(x)
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
-            B = S.Bernoulli(n+1)
-            F = Basic.Factorial(n+1)
+            B = C.bernoulli(n+1)
+            F = C.Factorial(n+1)
 
             return 2**(n+1) * B/F * x**n
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        denom = sinh(re)**2 + Basic.sin(im)**2
+        re, im = self.args[0].as_real_imag()
+        denom = sinh(re)**2 + C.sin(im)**2
         return (sinh(re)*cosh(re) - \
-            S.ImaginaryUnit*Basic.sin(im)*Basic.cos(im))/denom
+            S.ImaginaryUnit*C.sin(im)*C.cos(im))/denom
 
     def _eval_rewrite_as_exp(self, arg):
         neg_exp, pos_exp = S.Exp(-arg), S.Exp(arg)
@@ -372,9 +382,9 @@ class coth(Function):
         return 1/tanh(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
@@ -390,7 +400,7 @@ class asinh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return (self[0]**2 + 1)**(-S.Half)
+            return (self.args[0]**2 + 1)**(-S.Half)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -400,40 +410,41 @@ class asinh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeInfinity
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
-            elif isinstance(arg, Basic.One):
-                return S.Log(S.Sqrt(2) + 2)
-            elif isinstance(arg, Basic.NegativeOne):
-                return S.Log(S.Sqrt(2) - 2)
+            elif arg is S.One:
+                return C.log(2**S.Half + 1)
+            elif arg is S.NegativeOne:
+                return C.log(2**S.Half - 1)
             elif arg.is_negative:
                 return -cls(-arg)
         else:
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * S.ASin(i_coeff)
+                return S.ImaginaryUnit * C.asin(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
@@ -441,15 +452,15 @@ class asinh(Function):
             else:
                 k = (n - 1) // 2
 
-                R = Basic.RisingFactorial(S.Half, k)
-                F = Basic.Factorial(k)
+                R = C.RisingFactorial(S.Half, k)
+                F = C.Factorial(k)
 
                 return (-1)**k * R / F * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
@@ -460,7 +471,7 @@ class acosh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return (self[0]**2 - 1)**(-S.Half)
+            return (self.args[0]**2 - 1)**(-S.Half)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -470,44 +481,45 @@ class acosh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeInfinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Pi*S.ImaginaryUnit / 2
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Zero
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return S.Pi*S.ImaginaryUnit
             else:
                 cst_table = {
                     S.Half       : S.Pi/3,
                     -S.Half      : 2*S.Pi/3,
-                    S.Sqrt(2)/2  : S.Pi/4,
-                    -S.Sqrt(2)/2 : 3*S.Pi/4,
-                    1/S.Sqrt(2)  : S.Pi/4,
-                    -1/S.Sqrt(2) : 3*S.Pi/4,
-                    S.Sqrt(3)/2  : S.Pi/6,
-                    -S.Sqrt(3)/2 : 5*S.Pi/6,
+                    sqrt(2)/2    : S.Pi/4,
+                    -sqrt(2)/2   : 3*S.Pi/4,
+                    1/sqrt(2)    : S.Pi/4,
+                    -1/sqrt(2)   : 3*S.Pi/4,
+                    sqrt(3)/2    : S.Pi/6,
+                    -sqrt(3)/2   : 5*S.Pi/6,
                 }
 
                 if arg in cst_table:
                     return cst_table[arg]*S.ImaginaryUnit
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
             return S.Pi*S.ImaginaryUnit / 2
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
@@ -515,15 +527,15 @@ class acosh(Function):
             else:
                 k = (n - 1) // 2
 
-                R = Basic.RisingFactorial(S.Half, k)
-                F = Basic.Factorial(k)
+                R = C.RisingFactorial(S.Half, k)
+                F = C.Factorial(k)
 
                 return -R / F * S.ImaginaryUnit * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
@@ -534,7 +546,7 @@ class atanh(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/(1-self[0]**2)
+            return 1/(1-self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -544,16 +556,16 @@ class atanh(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Infinity
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return S.NegativeInfinity
             elif arg.is_negative:
                 return -cls(-arg)
@@ -561,25 +573,26 @@ class atanh(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * S.ATan(i_coeff)
+                return S.ImaginaryUnit * C.atan(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
             return x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
@@ -590,7 +603,7 @@ class acoth(Function):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/(1-self[0]**2)
+            return 1/(1-self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -600,20 +613,20 @@ class acoth(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
+        arg = sympify(arg)
 
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Zero
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.Zero
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Pi*S.ImaginaryUnit / 2
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Infinity
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return S.NegativeInfinity
             elif arg.is_negative:
                 return -cls(-arg)
@@ -621,27 +634,28 @@ class acoth(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return -S.ImaginaryUnit * S.ACot(i_coeff)
+                return -S.ImaginaryUnit * C.acot(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
                 if coeff.is_negative:
                     return -cls(-arg)
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
             return S.Pi*S.ImaginaryUnit / 2
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
             return x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
