@@ -1,18 +1,54 @@
 
-from sympy.core.basic import Basic, S, cache_it, cache_it_immutable
+from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Lambda, Function
+from miscellaneous import sqrt
+from sympy.core.cache import cacheit
 
 ###############################################################################
 ########################## TRIGONOMETRIC FUNCTIONS ############################
 ###############################################################################
 
 class sin(Function):
+    """
+    Usage
+    =====
+      sin(x) -> Returns the sine of x (measured in radians)
+
+    Notes
+    =====
+        sin(x) will evaluate automatically in the case x
+        is a multiple of pi, pi/2, pi/3, pi/4 and pi/6.
+
+    Examples
+    ========
+        >>> from sympy import *
+        >>> x = Symbol('x')
+        >>> sin(x**2).diff(x)
+        2*x*cos(x**2)
+        >>> sin(1).diff(x)
+        0
+        >>> sin(pi)
+        0
+        >>> sin(pi/2)
+        1
+        >>> sin(pi/6)
+        1/2
+
+    See also
+    ========
+       L{cos}, L{tan}
+
+       External links
+       --------------
+
+         U{Definitions in trigonometry<http://planetmath.org/encyclopedia/DefinitionsInTrigonometry.html>}
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return cos(self[0])
+            return cos(self.args[0])
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -25,12 +61,10 @@ class sin(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
             elif arg.is_negative:
                 return -cls(-arg)
@@ -38,18 +72,18 @@ class sin(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.sinh(i_coeff)
+                return S.ImaginaryUnit * C.sinh(i_coeff)
             else:
                 pi_coeff = arg.as_coefficient(S.Pi)
 
                 if pi_coeff is not None:
                     if pi_coeff.is_integer:
                         return S.Zero
-                    elif isinstance(pi_coeff, Basic.Rational):
+                    elif pi_coeff.is_Rational:
                         cst_table = {
                             2 : S.One,
-                            3 : S.Half*Basic.sqrt(3),
-                            4 : S.Half*Basic.sqrt(2),
+                            3 : S.Half*sqrt(3),
+                            4 : S.Half*sqrt(2),
                             6 : S.Half,
                         }
 
@@ -69,22 +103,22 @@ class sin(Function):
                     return -cls(-arg)
 
 
-    @classmethod
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
                 return -p * x**2 / (n*(n-1))
             else:
-                return (-1)**(n//2) * x**(n)/Basic.Factorial(n)
+                return (-1)**(n//2) * x**(n)/C.Factorial(n)
 
     def _eval_rewrite_as_exp(self, arg):
-        exp, I = Basic.exp, S.ImaginaryUnit
+        exp, I = C.exp, S.ImaginaryUnit
         return (exp(arg*I) - exp(-arg*I)) / (2*I)
 
     def _eval_rewrite_as_cos(self, arg):
@@ -99,56 +133,90 @@ class sin(Function):
         return 2*cot_half/(1 + cot_half**2)
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        return sin(re)*Basic.cosh(im) + S.ImaginaryUnit*cos(re)*Basic.sinh(im)
+        re, im = self.args[0].as_real_imag()
+        return sin(re)*C.cosh(im) + S.ImaginaryUnit*cos(re)*C.sinh(im)
 
     def _eval_expand_trig(self, *args):
-        arg = self[0].expand()
+        arg = self.args[0].expand()
         x = None
-        if isinstance(arg, Basic.Add):
-            x = arg[0]
-            y = Basic.Add(*arg[1:])
+        if arg.is_Add:
+            x = arg.args[0]
+            y = C.Add(*arg.args[1:])
         else:
             coeff, terms = arg.as_coeff_terms()
-            if not isinstance(coeff, Basic.One) and isinstance(coeff, Basic.Integer) and terms:
-                x = Basic.Mul(*terms)
+            if not (coeff is S.One) and coeff.is_Integer and terms:
+                x = C.Mul(*terms)
                 y = (coeff-1)*x
         if x is not None:
             return (sin(x)*cos(y) + sin(y)*cos(x)).expand(trig=True)
         return sin(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
     def _eval_is_real(self):
-        return self[0].is_real
+        return self.args[0].is_real
 
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
         if arg.is_real:
             return True
 
     def _sage_(self):
         import sage.all as sage
-        return sage.sin(self[0]._sage_())
+        return sage.sin(self.args[0]._sage_())
 
 class cos(Function):
+    """
+    Usage
+    =====
+      cos(x) -> Returns the cosine of x (measured in radians)
+
+    Notes
+    =====
+        cos(x) will evaluate automatically in the case x
+        is a multiple of pi, pi/2, pi/3, pi/4 and pi/6.
+
+    Examples
+    ========
+        >>> from sympy import *
+        >>> x = Symbol('x')
+        >>> cos(x**2).diff(x)
+        -2*sin(x**2)*x
+        >>> cos(1).diff(x)
+        0
+        >>> cos(pi)
+        -1
+        >>> cos(pi/2)
+        0
+        >>> cos(2*pi/3)
+        -1/2
+
+    See also
+    ========
+       L{sin}, L{tan}
+
+       External links
+       --------------
+
+         U{Definitions in trigonometry<http://planetmath.org/encyclopedia/DefinitionsInTrigonometry.html>}
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return -sin(self[0])
+            return -sin(self.args[0])
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -161,12 +229,10 @@ class cos(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.One
             elif arg.is_negative:
                 return cls(-arg)
@@ -174,18 +240,18 @@ class cos(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return Basic.cosh(i_coeff)
+                return C.cosh(i_coeff)
             else:
                 pi_coeff = arg.as_coefficient(S.Pi)
 
                 if pi_coeff is not None:
-                    if isinstance(pi_coeff, Basic.Rational):
+                    if pi_coeff.is_Rational:
                         cst_table = {
                             1 : S.One,
                             2 : S.Zero,
                             3 : S.Half,
-                            4 : S.Half*Basic.sqrt(2),
-                            6 : S.Half*Basic.sqrt(3),
+                            4 : S.Half*sqrt(2),
+                            6 : S.Half*sqrt(3),
                         }
 
                         try:
@@ -204,22 +270,22 @@ class cos(Function):
                     return cls(-arg)
 
 
-    @classmethod
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 1:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
                 return -p * x**2 / (n*(n-1))
             else:
-                return (-1)**(n//2)*x**(n)/Basic.Factorial(n)
+                return (-1)**(n//2)*x**(n)/C.Factorial(n)
 
     def _eval_rewrite_as_exp(self, arg):
-        exp, I = Basic.exp, S.ImaginaryUnit
+        exp, I = C.exp, S.ImaginaryUnit
         return (exp(arg*I) + exp(-arg*I)) / 2
 
     def _eval_rewrite_as_sin(self, arg):
@@ -234,48 +300,79 @@ class cos(Function):
         return (cot_half-1)/(cot_half+1)
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        return cos(re)*Basic.cosh(im) - \
-            S.ImaginaryUnit*sin(re)*Basic.sinh(im)
+        re, im = self.args[0].as_real_imag()
+        return cos(re)*C.cosh(im) - \
+            S.ImaginaryUnit*sin(re)*C.sinh(im)
 
     def _eval_expand_trig(self, *args):
-        arg = self[0].expand()
+        arg = self.args[0].expand()
         x = None
-        if isinstance(arg, Basic.Add):
-            x = arg[0]
-            y = Basic.Add(*arg[1:])
+        if arg.is_Add:
+            x = arg.args[0]
+            y = C.Add(*arg.args[1:])
             return (cos(x)*cos(y) - sin(y)*sin(x)).expand(trig=True)
         else:
             coeff, terms = arg.as_coeff_terms()
-            if not isinstance(coeff, Basic.One) and isinstance(coeff, Basic.Integer) and terms:
-                x = Basic.Mul(*terms)
-                return Basic.chebyshevt(coeff, cos(x))
+            if not (coeff is S.One) and coeff.is_Integer and terms:
+                x = C.Mul(*terms)
+                return C.chebyshevt(coeff, cos(x))
         return cos(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real
+
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
 
         if arg.is_real:
             return True
 
     def _sage_(self):
         import sage.all as sage
-        return sage.cos(self[0]._sage_())
+        return sage.cos(self.args[0]._sage_())
 
 class tan(Function):
+    """
+    Usage
+    =====
+      tan(x) -> Returns the tangent of x (measured in radians)
+
+    Notes
+    =====
+        tan(x) will evaluate automatically in the case x is a
+        multiple of pi.
+
+    Examples
+    ========
+        >>> from sympy import *
+        >>> x = Symbol('x')
+        >>> tan(x**2).diff(x)
+        2*x*cos(x**2)**(-2)
+        >>> tan(1).diff(x)
+        0
+
+    See also
+    ========
+       L{sin}, L{tan}
+
+       External links
+       --------------
+
+         U{Definitions in trigonometry<http://planetmath.org/encyclopedia/DefinitionsInTrigonometry.html>}
+    """
 
     nargs = 1
 
@@ -294,12 +391,10 @@ class tan(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
             elif arg.is_negative:
                 return -cls(-arg)
@@ -307,19 +402,19 @@ class tan(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.tanh(i_coeff)
+                return S.ImaginaryUnit * C.tanh(i_coeff)
             else:
                 pi_coeff = arg.as_coefficient(S.Pi)
 
                 if pi_coeff is not None:
                     if pi_coeff.is_integer:
                         return S.Zero
-                    elif isinstance(pi_coeff, Basic.Rational):
+                    elif pi_coeff.is_Rational:
                         cst_table = {
                            #2 : S.ComplexInfinity,
-                            3 : Basic.sqrt(3),
+                            3 : sqrt(3),
                             4 : S.One,
-                            6 : 1 / Basic.sqrt(3),
+                            6 : 1 / sqrt(3),
                         }
 
                         try:
@@ -338,37 +433,37 @@ class tan(Function):
                     return -cls(-arg)
 
 
-    @classmethod
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             a, b = ((n-1)//2), 2**(n+1)
 
-            B = Basic.bernoulli(n+1)
-            F = Basic.Factorial(n+1)
+            B = C.bernoulli(n+1)
+            F = C.Factorial(n+1)
 
             return (-1)**a * b*(b-1) * B/F * x**n
 
     def _eval_conjugate(self):
-        return self.func(self[0].conjugate())
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        denom = cos(re)**2 + Basic.sinh(im)**2
+        re, im = self.args[0].as_real_imag()
+        denom = cos(re)**2 + C.sinh(im)**2
         return (sin(re)*cos(re) + \
-            S.ImaginaryUnit*Basic.sinh(im)*Basic.cosh(im))/denom
+            S.ImaginaryUnit*C.sinh(im)*C.cosh(im))/denom
 
     def _eval_expand_trig(self, *args):
         return self
 
     def _eval_rewrite_as_exp(self, arg):
-        exp, I = Basic.exp, S.ImaginaryUnit
+        exp, I = C.exp, S.ImaginaryUnit
         neg_exp, pos_exp = exp(-arg*I), exp(arg*I)
         return I*(neg_exp-pos_exp)/(neg_exp+pos_exp)
 
@@ -382,20 +477,28 @@ class tan(Function):
         return 1/S.Cot(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real
+
     def _eval_is_bounded(self):
-        arg = self[0]
+        arg = self.args[0]
 
         if arg.is_imaginary:
             return True
 
 class cot(Function):
+    """
+    Usage
+    =====
+      cot(x) -> Returns the cotangent of x (measured in radians)
+    """
 
     nargs = 1
 
@@ -414,12 +517,10 @@ class cot(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            #elif isinstance(arg, Basic.Zero):
+            #elif arg is S.Zero:
             #    return S.ComplexInfinity
             elif arg.is_negative:
                 return -cls(-arg)
@@ -427,19 +528,19 @@ class cot(Function):
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return -S.ImaginaryUnit * Basic.coth(i_coeff)
+                return -S.ImaginaryUnit * C.coth(i_coeff)
             else:
                 pi_coeff = arg.as_coefficient(S.Pi)
 
                 if pi_coeff is not None:
                     #if pi_coeff.is_integer:
                     #    return S.ComplexInfinity
-                    if isinstance(pi_coeff, Basic.Rational):
+                    if pi_coeff.is_Rational:
                         cst_table = {
                             2 : S.Zero,
-                            3 : 1 / Basic.sqrt(3),
+                            3 : 1 / sqrt(3),
                             4 : S.One,
-                            6 : Basic.sqrt(3)
+                            6 : sqrt(3)
                         }
 
                         try:
@@ -458,34 +559,35 @@ class cot(Function):
                     return -cls(-arg)
 
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
-            return 1 / Basic.sympify(x)
+            return 1 / sympify(x)
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             B = S.Bernoulli(n+1)
-            F = Basic.Factorial(n+1)
+            F = C.Factorial(n+1)
 
             return (-1)**((n+1)//2) * 2**(n+1) * B/F * x**n
 
     def _eval_conjugate(self):
-        assert len(self) == 1
-        return self.func(self[0].conjugate())
+        assert len(self.args) == 1
+        return self.func(self.args[0].conjugate())
 
     def _eval_expand_complex(self, *args):
-        if self[0].is_real:
+        if self.args[0].is_real:
             return self
-        re, im = self[0].as_real_imag()
-        denom = sin(re)**2 + Basic.sinh(im)**2
+        re, im = self.args[0].as_real_imag()
+        denom = sin(re)**2 + C.sinh(im)**2
         return (sin(re)*cos(re) - \
-            S.ImaginaryUnit*Basic.sinh(im)*Basic.cosh(im))/denom
+            S.ImaginaryUnit*C.sinh(im)*C.cosh(im))/denom
 
     def _eval_rewrite_as_exp(self, arg):
-        exp, I = Basic.exp, S.ImaginaryUnit
+        exp, I = C.exp, S.ImaginaryUnit
         neg_exp, pos_exp = exp(-arg*I), exp(arg*I)
         return I*(pos_exp+neg_exp)/(pos_exp-neg_exp)
 
@@ -499,24 +601,32 @@ class cot(Function):
         return 1/tan(arg)
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return S.One
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real
+
 ###############################################################################
 ########################### TRIGONOMETRIC INVERSES ############################
 ###############################################################################
 
 class asin(Function):
+    """
+    Usage
+    =====
+      asin(x) -> Returns the arc sine of x (measured in radians)
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return (1 - self[0]**2)**(-S.Half)
+            return (1 - self.args[0]**2)**(-S.Half)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -526,42 +636,41 @@ class asin(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.NegativeInfinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.Infinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Pi / 2
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return -S.Pi / 2
-            else:
-                cst_table = {
-                    S.Half       : 6,
-                    -S.Half      : -6,
-                    Basic.sqrt(2)/2  : 4,
-                    -Basic.sqrt(2)/2 : -4,
-                    1/Basic.sqrt(2)  : 4,
-                    -1/Basic.sqrt(2) : -4,
-                    Basic.sqrt(3)/2  : 3,
-                    -Basic.sqrt(3)/2 : -3,
+
+        if arg.is_number:
+            cst_table = {
+                S.Half     : 6,
+                -S.Half    : -6,
+                sqrt(2)/2  : 4,
+                -sqrt(2)/2 : -4,
+                1/sqrt(2)  : 4,
+                -1/sqrt(2) : -4,
+                sqrt(3)/2  : 3,
+                -sqrt(3)/2 : -3,
                 }
 
-                if arg in cst_table:
-                    return S.Pi / cst_table[arg]
-                elif arg.is_negative:
-                    return -cls(-arg)
+            if arg in cst_table:
+                return S.Pi / cst_table[arg]
+            elif arg.is_negative:
+                return -cls(-arg)
         else:
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.asinh(i_coeff)
+                return S.ImaginaryUnit * C.asinh(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
@@ -569,12 +678,13 @@ class asin(Function):
                     return -cls(-arg)
 
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
@@ -582,26 +692,34 @@ class asin(Function):
             else:
                 k = (n - 1) // 2
 
-                R = Basic.RisingFactorial(S.Half, k)
-                F = Basic.Factorial(k)
+                R = C.RisingFactorial(S.Half, k)
+                F = C.Factorial(k)
 
                 return R / F * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real and (self.args[0]>=-1 and self.args[0]<=1)
+
 class acos(Function):
+    """
+    Usage
+    =====
+      acos(x) -> Returns the arc cosine of x (measured in radians)
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return -(1 - self[0]**2)**(-S.Half)
+            return -(1 - self.args[0]**2)**(-S.Half)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -611,45 +729,45 @@ class acos(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Infinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.NegativeInfinity * S.ImaginaryUnit
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Pi / 2
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Zero
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return S.Pi
-            else:
-                cst_table = {
-                    S.Half       : S.Pi/3,
-                    -S.Half      : 2*S.Pi/3,
-                    Basic.sqrt(2)/2  : S.Pi/4,
-                    -Basic.sqrt(2)/2 : 3*S.Pi/4,
-                    1/Basic.sqrt(2)  : S.Pi/4,
-                    -1/Basic.sqrt(2) : 3*S.Pi/4,
-                    Basic.sqrt(3)/2  : S.Pi/6,
-                    -Basic.sqrt(3)/2 : 5*S.Pi/6,
+
+        if arg.is_number:
+            cst_table = {
+                S.Half     : S.Pi/3,
+                -S.Half    : 2*S.Pi/3,
+                sqrt(2)/2  : S.Pi/4,
+                -sqrt(2)/2 : 3*S.Pi/4,
+                1/sqrt(2)  : S.Pi/4,
+                -1/sqrt(2) : 3*S.Pi/4,
+                sqrt(3)/2  : S.Pi/6,
+                -sqrt(3)/2 : 5*S.Pi/6,
                 }
 
-                if arg in cst_table:
-                    return cst_table[arg]
+            if arg in cst_table:
+                return cst_table[arg]
 
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
             return S.Pi / 2
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
@@ -657,26 +775,34 @@ class acos(Function):
             else:
                 k = (n - 1) // 2
 
-                R = Basic.RisingFactorial(S.Half, k)
-                F = Basic.Factorial(k)
+                R = C.RisingFactorial(S.Half, k)
+                F = C.Factorial(k)
 
                 return -R / F * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real and (self.args[0]>=-1 and self.args[0]<=1)
+
 class atan(Function):
+    """
+    Usage
+    =====
+      tan(x) -> Returns the arc tangent of x (measured in radians)
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/(1+self[0]**2)
+            return 1/(1+self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -686,40 +812,40 @@ class atan(Function):
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Pi / 2
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return -S.Pi / 2
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Zero
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Pi / 4
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return -S.Pi / 4
-            else:
-                cst_table = {
-                    Basic.sqrt(3)/3  : 6,
-                    -Basic.sqrt(3)/3 : -6,
-                    1/Basic.sqrt(3)  : 6,
-                    -1/Basic.sqrt(3) : -6,
-                    Basic.sqrt(3)    : 3,
-                    -Basic.sqrt(3)   : -3,
+
+        if arg.is_number:
+            cst_table = {
+                sqrt(3)/3  : 6,
+                -sqrt(3)/3 : -6,
+                1/sqrt(3)  : 6,
+                -1/sqrt(3) : -6,
+                sqrt(3)    : 3,
+                -sqrt(3)   : -3,
                 }
 
-                if arg in cst_table:
-                    return S.Pi / cst_table[arg]
-                elif arg.is_negative:
-                    return -cls(-arg)
+            if arg in cst_table:
+                return S.Pi / cst_table[arg]
+            elif arg.is_negative:
+                return -cls(-arg)
+
         else:
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return S.ImaginaryUnit * Basic.atanh(i_coeff)
+                return S.ImaginaryUnit * C.atanh(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
@@ -727,68 +853,77 @@ class atan(Function):
                     return -cls(-arg)
 
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
             return (-1)**((n-1)//2) * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
 
+    def _eval_is_real(self):
+        return self.args[0].is_real
+
 class acot(Function):
+    """
+    Usage
+    =====
+      acot(x) -> Returns the arc cotangent of x (measured in radians)
+    """
 
     nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return -1 / (1+self[0]**2)
+            return -1 / (1+self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
     def canonize(cls, arg):
-        arg = Basic.sympify(arg)
-
-        if isinstance(arg, Basic.Number):
-            if isinstance(arg, Basic.NaN):
+        if arg.is_Number:
+            if arg is S.NaN:
                 return S.NaN
-            elif isinstance(arg, Basic.Infinity):
+            elif arg is S.Infinity:
                 return S.Zero
-            elif isinstance(arg, Basic.NegativeInfinity):
+            elif arg is S.NegativeInfinity:
                 return S.Zero
-            elif isinstance(arg, Basic.Zero):
+            elif arg is S.Zero:
                 return S.Pi/ 2
-            elif isinstance(arg, Basic.One):
+            elif arg is S.One:
                 return S.Pi / 4
-            elif isinstance(arg, Basic.NegativeOne):
+            elif arg is S.NegativeOne:
                 return -S.Pi / 4
-            else:
-                cst_table = {
-                    Basic.sqrt(3)/3  : 3,
-                    -Basic.sqrt(3)/3 : -3,
-                    1/Basic.sqrt(3)  : 3,
-                    -1/Basic.sqrt(3) : -3,
-                    Basic.sqrt(3)    : 6,
-                    -Basic.sqrt(3)   : -6,
+
+        if arg.is_number:
+            cst_table = {
+                sqrt(3)/3  : 3,
+                -sqrt(3)/3 : -3,
+                1/sqrt(3)  : 3,
+                -1/sqrt(3) : -3,
+                sqrt(3)    : 6,
+                -sqrt(3)   : -6,
                 }
 
-                if arg in cst_table:
-                    return S.Pi / cst_table[arg]
-                elif arg.is_negative:
-                    return -cls(-arg)
+            if arg in cst_table:
+                return S.Pi / cst_table[arg]
+            elif arg.is_negative:
+                return -cls(-arg)
+
         else:
             i_coeff = arg.as_coefficient(S.ImaginaryUnit)
 
             if i_coeff is not None:
-                return -S.ImaginaryUnit * Basic.acoth(i_coeff)
+                return -S.ImaginaryUnit * C.acoth(i_coeff)
             else:
                 coeff, terms = arg.as_coeff_terms()
 
@@ -796,20 +931,59 @@ class acot(Function):
                     return -cls(-arg)
 
 
-    @cache_it_immutable
-    def taylor_term(self, n, x, *previous_terms):
+    @staticmethod
+    @cacheit
+    def taylor_term(n, x, *previous_terms):
         if n == 0:
             return S.Pi / 2 # FIX THIS
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
-            x = Basic.sympify(x)
+            x = sympify(x)
             return (-1)**((n+1)//2) * x**n / n
 
     def _eval_as_leading_term(self, x):
-        arg = self[0].as_leading_term(x)
+        arg = self.args[0].as_leading_term(x)
 
-        if Basic.Order(1,x).contains(arg):
+        if C.Order(1,x).contains(arg):
             return arg
         else:
             return self.func(arg)
+
+    def _eval_is_real(self):
+        return self.args[0].is_real
+
+class atan2(Function):
+    """
+    Returns the atan(y/x) taking two arguments y and x. Signs of
+    both y and x are considered to determine the appropriate
+    quadrant of atan(y/x). The range is (-pi, pi].
+    """
+
+    nargs = 2
+
+    @classmethod
+    def canonize(cls, y, x):
+        sign_y = C.sign(y)
+
+        if y.is_zero:
+            if x.is_positive:
+                return S.Zero
+            elif x.is_zero:
+                return S.NaN
+            elif x.is_negative:
+                return S.Pi
+        elif x.is_zero:
+            if sign_y.is_Number:
+                return sign_y * S.Pi/2
+        else:
+            abs_yx = C.abs(y/x)
+            if sign_y.is_Number and abs_yx.is_number:
+                phi = C.atan(abs_yx)
+                if x.is_positive:
+                    return sign_y * phi
+                else:
+                    return sign_y * (S.Pi - phi)
+
+    def _eval_is_real(self):
+        return self.args[0].is_real and self.args[1].is_real
