@@ -1,5 +1,7 @@
+from sympy import Basic, Symbol
+from sympy.core import sympify
 
-from sympy import *
+from sympy.core.basic import S, C
 
 import sympy.polynomials
 import random
@@ -11,6 +13,25 @@ class ShapeError(ValueError):
     """Wrong matrix shape"""
     pass
 
+class MatrixError(Exception):
+    pass
+
+class _MatrixAsBasic(Basic):
+    """Proxy between Matrix & Basic
+
+    The reason, this class is here, is that Matrix does not inherit from Basic.
+    So for common functionality implemented in Basic, we have to provide this
+    proxy.
+
+    see #420
+"""
+    def __init__(self, m):
+        self.m = m
+    def torepr(self):
+        return self.m.torepr()
+    def tostr(self):
+        return self.m.tostr()
+
 class Matrix(object):
 
     def __init__(self, *args):
@@ -19,11 +40,11 @@ class Matrix(object):
 
         >>> from sympy import *
         >>> Matrix( (1,2+I), (3,4) ) #doctest:+NORMALIZE_WHITESPACE
-        1 2 + I
-        3 4
+        [1, 2 + I]
+        [3,     4]
         >>> Matrix(2, 2, lambda i,j: (i+1)*j ) #doctest:+NORMALIZE_WHITESPACE
-        0 1
-        0 2
+        [0, 1]
+        [0, 2]
 
         Note: in SymPy we count indices from 0. The rule however counts from 1.
         """
@@ -35,7 +56,7 @@ class Matrix(object):
             self.mat = []
             for i in range(self.lines):
                 for j in range(self.cols):
-                    self.mat.append(Basic.sympify(operation(i, j)))
+                    self.mat.append(sympify(operation(i, j)))
         elif len(args)==3 and isinstance(args[0],int) and \
                 isinstance(args[1],int) and isinstance(args[2], (list, tuple)):
             self.lines=args[0]
@@ -44,13 +65,18 @@ class Matrix(object):
             self.mat=[]
             for j in range(self.lines):
                 for i in range(self.cols):
-                    self.mat.append(Basic.sympify(mat[j*self.cols+i]))
+                    self.mat.append(sympify(mat[j*self.cols+i]))
         else:
             if len(args) == 1:
                 mat = args[0]
             else:
                 mat = args
-            if not isinstance(mat[0], (list, tuple)):
+            if hasattr(mat, "__array__"):
+                # NumPy array or matrix or some other object that implements
+                # __array__ (like SymPy Matrix). So let's first use this method
+                # to get a numpy.array() and then make a python list out of it.
+                mat = list(mat.__array__())
+            elif not isinstance(mat[0], (list, tuple)):
                 # make each element a singleton
                 mat = [ [element] for element in mat ]
             self.lines=len(mat)
@@ -59,7 +85,7 @@ class Matrix(object):
             for j in range(self.lines):
                 assert len(mat[j])==self.cols
                 for i in range(self.cols):
-                    self.mat.append(Basic.sympify(mat[j][i]))
+                    self.mat.append(sympify(mat[j][i]))
 
     def key2ij(self,key):
         """Converts key=(4,6) to 4,6 and ensures the key is correct."""
@@ -77,14 +103,14 @@ class Matrix(object):
         >>> from sympy import *
         >>> m=Matrix(((1,2+I),(3,4)))
         >>> m  #doctest: +NORMALIZE_WHITESPACE
-        1 2 + I
-        3 4
+        [1, 2 + I]
+        [3,     4]
         >>> m.T #doctest: +NORMALIZE_WHITESPACE
-        1 3
-        2 + I 4
+        [    1, 3]
+        [2 + I, 4]
         >>> m.H #doctest: +NORMALIZE_WHITESPACE
-        1 3
-        2 - I 4
+        [    1, 3]
+        [2 - I, 4]
 
         """
         if name == "T":
@@ -113,8 +139,8 @@ class Matrix(object):
         >>> from sympy import *
         >>> m=Matrix(((1,2+I),(3,4)))
         >>> m  #doctest: +NORMALIZE_WHITESPACE
-        1 2 + I
-        3 4
+        [1, 2 + I]
+        [3,     4]
         >>> m[1,0]
         3
         >>> m.H[1,0]
@@ -139,12 +165,12 @@ class Matrix(object):
         >>> from sympy import *
         >>> m=Matrix(((1,2+I),(3,4)))
         >>> m  #doctest: +NORMALIZE_WHITESPACE
-        1 2 + I
-        3 4
+        [1, 2 + I]
+        [3,     4]
         >>> m[1,0]=9
         >>> m  #doctest: +NORMALIZE_WHITESPACE
-        1 2 + I
-        9 4
+        [1, 2 + I]
+        [9,     4]
 
         """
         assert len(key) == 2
@@ -155,7 +181,11 @@ class Matrix(object):
                 self.copyin_list(key, value)
         else:
             i,j=self.key2ij(key)
-            self.mat[i*self.cols + j] = Basic.sympify(value)
+            self.mat[i*self.cols + j] = sympify(value)
+
+    def __array__(self):
+        return matrix2numpy(self)
+
 
     def copyin_matrix(self, key, value):
         rlo, rhi = self.slice2bounds(key[0], self.lines)
@@ -163,7 +193,7 @@ class Matrix(object):
         assert value.lines == rhi - rlo and value.cols == chi - clo
         for i in range(value.lines):
             for j in range(value.cols):
-                self[i+rlo, j+clo] = Basic.sympify(value[i,j])
+                self[i+rlo, j+clo] = sympify(value[i,j])
 
     def copyin_list(self, key, value):
         assert isinstance(value, (list, tuple))
@@ -232,7 +262,7 @@ class Matrix(object):
         return self.add(a)
 
     def __div__(self,a):
-        return self * (Rational(1)/a)
+        return self * (S.One/a)
 
     def multiply(self,b):
         """Returns self*b """
@@ -262,16 +292,47 @@ class Matrix(object):
 
     def __eq__(self,a):
         if not isinstance(a, (Matrix, Basic)):
-            a = Basic.sympify(a)
+            a = sympify(a)
         return self.hash() == a.hash()
 
     def __ne__(self,a):
         if not isinstance(a, (Matrix, Basic)):
-            a = Basic.sympify(a)
+            a = sympify(a)
         return self.hash() != a.hash()
 
+    # hook Basic.__repr__ & Basic.__str__
     def __repr__(self):
-        return str(self)
+        return _MatrixAsBasic(self).__repr__()
+
+    def __str__(self):
+        return _MatrixAsBasic(self).__str__()
+
+    def _format_str(self, strfunc, rowsep='\n'):
+        # Build table of string representations of the elements
+        res = []
+        # Track per-column max lengths for pretty alignment
+        maxlen = [0] * self.cols
+        for i in range(self.lines):
+            res.append([])
+            for j in range(self.cols):
+                string = strfunc(self[i,j])
+                res[-1].append(string)
+                maxlen[j] = max(len(string), maxlen[j])
+        # Patch strings together
+        for i, row in enumerate(res):
+            for j, elem in enumerate(row):
+                # Pad each element up to maxlen so the columns line up
+                row[j] = elem.rjust(maxlen[j])
+            res[i] = "[" + ", ".join(row) + "]"
+        return rowsep.join(res)
+
+    def torepr(self):
+        s = self._format_str(lambda elem: repr(elem), ',\n  ')
+
+        return '%s([\n  %s,\n])' % (self.__class__.__name__, s)
+
+    def tostr(self):
+        return self._format_str(lambda elem: str(elem))
 
     def inv(self, method="GE"):
         assert self.cols==self.lines
@@ -283,14 +344,6 @@ class Matrix(object):
         else:
             raise "Inversion method unrecognized"
 
-    def __str__(self):
-        s="";
-        for i in range(self.lines):
-            for j in range(self.cols):
-                s+="%s "%repr(self[i,j]);
-            if i != self.lines - 1:
-                s+="\n"
-        return s
 
     def __mathml__(self):
         mml = ""
@@ -329,9 +382,9 @@ class Matrix(object):
         >>> M = sympy.matrices.eye(3)
         >>> M.col_del(1)
         >>> M   #doctest: +NORMALIZE_WHITESPACE
-        1 0
-        0 0
-        0 1
+        [1, 0]
+        [0, 0]
+        [0, 1]
         """
         for j in range(self.lines-1, -1, -1):
             del self.mat[i+j*self.cols]
@@ -364,18 +417,18 @@ class Matrix(object):
         >>> from sympy import *
         >>> m = Matrix(4,4,lambda i,j: i+j)
         >>> m   #doctest: +NORMALIZE_WHITESPACE
-        0 1 2 3
-        1 2 3 4
-        2 3 4 5
-        3 4 5 6
+        [0, 1, 2, 3]
+        [1, 2, 3, 4]
+        [2, 3, 4, 5]
+        [3, 4, 5, 6]
         >>> m[0:1, 1]   #doctest: +NORMALIZE_WHITESPACE
-        1
+        [1]
         >>> m[0:2, 0:1] #doctest: +NORMALIZE_WHITESPACE
-        0
-        1
+        [0]
+        [1]
         >>> m[2:4, 2:4] #doctest: +NORMALIZE_WHITESPACE
-        4 5
-        5 6
+        [4, 5]
+        [5, 6]
         """
         assert isinstance(keys[0], slice) or isinstance(keys[1], slice)
         rlo, rhi = self.slice2bounds(keys[0], self.lines)
@@ -415,11 +468,11 @@ class Matrix(object):
         >>> from sympy import *
         >>> m = Matrix(2,2,lambda i,j: i*2+j)
         >>> m   #doctest: +NORMALIZE_WHITESPACE
-        0 1
-        2 3
+        [0, 1]
+        [2, 3]
         >>> m.applyfunc(lambda i: 2*i)  #doctest: +NORMALIZE_WHITESPACE
-        0 2
-        4 6
+        [0, 2]
+        [4, 6]
         """
         assert callable(f)
         out = self[:,:]
@@ -433,14 +486,14 @@ class Matrix(object):
         >>> from sympy import *
         >>> m = Matrix(2,3,lambda i,j: 1)
         >>> m   #doctest: +NORMALIZE_WHITESPACE
-        1 1 1
-        1 1 1
+        [1, 1, 1]
+        [1, 1, 1]
         >>> m.reshape(1,6)  #doctest: +NORMALIZE_WHITESPACE
-        1 1 1 1 1 1
+        [1, 1, 1, 1, 1, 1]
         >>> m.reshape(3,2)  #doctest: +NORMALIZE_WHITESPACE
-        1 1
-        1 1
-        1 1
+        [1, 1]
+        [1, 1]
+        [1, 1]
         """
         if self.lines*self.cols != _rows*_cols:
             print "Invalid reshape parameters %d %d" % (_rows, _cols)
@@ -452,8 +505,8 @@ class Matrix(object):
         >>> from sympy import *
         >>> m = Matrix(2,3,lambda i,j: i*3+j)
         >>> m           #doctest: +NORMALIZE_WHITESPACE
-        0 1 2
-        3 4 5
+        [0, 1, 2]
+        [3, 4, 5]
         >>> m.print_nonzero()   #doctest: +NORMALIZE_WHITESPACE
         [ XX]
         [XXX]
@@ -686,10 +739,10 @@ class Matrix(object):
 
     def norm(self):
         assert self.lines == 1 or self.cols == 1
-        out = Basic.sympify(0)
+        out = sympify(0)
         for i in range(self.lines * self.cols):
             out += self[i]*self[i]
-        return out**Basic.Half()
+        return out**S.Half
 
     def normalized(self):
         assert self.lines == 1 or self.cols == 1
@@ -772,6 +825,7 @@ class Matrix(object):
 
            TODO: Implement algorithm for sparse matrices (SFF).
         """
+        from sympy.simplify import cancel
 
         if not self.is_square:
             raise NonSquareMatrixException()
@@ -795,7 +849,7 @@ class Matrix(object):
                             sign *= -1
                             break
                     else:
-                        return Rational(0)
+                        return S.Zero
 
                 # proceed with Bareis' fraction-free (FF)
                 # form of Gaussian elimination algorithm
@@ -804,9 +858,12 @@ class Matrix(object):
                         D = M[k, k]*M[i, j] - M[i, k]*M[k, j]
 
                         if k > 0:
-                            M[i, j] = D / M[k-1, k-1]
-                        else:
+                            D /= M[k-1, k-1]
+
+                        if D.is_Atom:
                             M[i, j] = D
+                        else:
+                            M[i, j] = cancel(D)
 
             det = sign * M[n-1, n-1]
 
@@ -884,6 +941,111 @@ class Matrix(object):
             copy[i,i] -= x
         return copy.det()
 
+    def berkowitz(self):
+        """The Berkowitz algorithm.
+
+           Given N x N matrix with symbolic content, compute efficiently
+           coefficients of characteristic polynomials of 'self' and all
+           its square sub-matrices composed by removing both i-th row
+           and column, without division in the ground domain.
+
+           This method is particulary useful for computing determinant,
+           principal minors and characteristic polynomial, when 'self'
+           has complicated coefficients eg. polynomials. Semi-direct
+           usage of this algorithm is also important in computing
+           efficiently subresultant PRS.
+
+           Assuming that M is a square matrix of dimension N x N and
+           I is N x N identity matrix,  then the following following
+           definition of characteristic polynomial is begin used:
+
+                          charpoly(M) = det(t*I - M)
+
+           As a consequence, all polynomials generated by Berkowitz
+           algorithm are monic.
+
+           >>> from sympy import *
+           >>> x,y,z = symbols('xyz')
+
+           >>> M = Matrix([ [x,y,z], [1,0,0], [y,z,x] ])
+
+           >>> p, q, r = M.berkowitz()
+
+           >>> print p # 1 x 1 M's sub-matrix
+           (1, -x)
+
+           >>> print q # 2 x 2 M's sub-matrix
+           (1, -x, -y)
+
+           >>> print r # 3 x 3 M's sub-matrix
+           (1, -2*x, -y - y*z + x**2, x*y - z**2)
+
+           For more information on the implemented algorithm refer to:
+
+           [1] S.J. Berkowitz, On computing the determinant in small
+               parallel time using a small number of processors, ACM,
+               Information Processing Letters 18, 1984, pp. 147-150
+
+           [2] M. Keber, Division-Free computation of subresultants
+               using Bezout matrices, Tech. Report MPI-I-2006-1-006,
+               Saarbrucken, 2006
+
+        """
+        if not self.is_square:
+            raise MatrixError
+
+        A, N = self, self.lines
+        transforms = [0] * (N-1)
+
+        for n in xrange(N, 1, -1):
+            T, k = zeronm(n+1,n), n - 1
+
+            R, C = -A[k,:k], A[:k,k]
+            A, a = A[:k,:k], -A[k,k]
+
+            items = [ C ]
+
+            for i in xrange(0, n-2):
+                items.append(A * items[i])
+
+            for i, B in enumerate(items):
+                items[i] = (R * B)[0,0]
+
+            items = [ S.One, a ] + items
+
+            for i in xrange(n):
+                T[i:,i] = items[:n-i+1]
+
+            transforms[k-1] = T
+
+        polys = [ Matrix(S.One, -A[0,0]) ]
+
+        for i, T in enumerate(transforms):
+            polys.append(T * polys[i])
+
+        return tuple(map(tuple, polys))
+
+    def berkowitz_det(self):
+        """Computes determinant using Berkowitz method."""
+        poly = self.berkowitz()[-1]
+        sign = (-1)**(len(poly)-1)
+        return sign * poly[-1]
+
+    def berkowitz_minors(self):
+        """Computes principal minors using Berkowitz method."""
+        sign, minors = S.NegativeOne, []
+
+        for poly in self.berkowitz():
+            minors.append(sign*poly[-1])
+            sign = -sign
+
+        return tuple(minors)
+
+    def berkowitz_charpoly(self, x):
+        """Computes characteristic polynomial minors using Berkowitz method."""
+        coeffs, monoms = self.berkowitz()[-1], range(self.lines+1)
+        return C.Poly(list(zip(coeffs, reversed(monoms))), x)
+
     def eigenvals(self, var=None):
         """ Calls polynomials's roots(), doesn't support coeff type right now """
         # returns list of pairs (eigenval, multiplicty)
@@ -926,20 +1088,24 @@ class Matrix(object):
         return out
 
 def zero(n):
+    """Create square zero matrix n x n"""
     return zeronm(n,n)
 
 def zeronm(n,m):
+    """Create zero matrix n x m"""
     assert n>0
     assert m>0
     return Matrix(n,m, lambda i,j: 0)
 
 def one(n):
+    """Create square all-one matrix n x n"""
     m = zero(n)
     for i in range(n):
         m[i,i]=1
     return m
 
 def eye(n):
+    """Create square identity matrix n x n"""
     assert n>0
     out = zeronm(n,n)
     for i in range(n):
@@ -947,13 +1113,18 @@ def eye(n):
     return out
 
 def randMatrix(r,c,min=0,max=99,seed=[]):
+    """Create random matrix r x c"""
     if seed == []:
-        random.seed()
+        prng = random.Random()  # use system time
     else:
-        random.seed(seed)       # use system time
-    return Matrix(r,c,lambda i,j: random.randint(min,max))
+        prng = random.Random(seed)
+    return Matrix(r,c,lambda i,j: prng.randint(min,max))
 
 def hessian(f, varlist):
+    """Compute Hessian matrix for a function f
+
+    see: http://en.wikipedia.org/wiki/Hessian_matrix
+    """
     # f is the expression representing a function f, return regular matrix
     if isinstance(varlist, (list, tuple)):
         m = len(varlist)
@@ -992,8 +1163,21 @@ def GramSchmidt(vlist, orthog=False):
     return out
 
 def wronskian(functions, var):
+    """Compute wronskian for [] of functions
+
+                   | f1    f2     ...   fn  |
+                   | f1'   f2'    ...   fn' |
+                   |  .     .     .      .  |
+    W(f1,...,fn) = |  .     .      .     .  |
+                   |  .     .       .    .  |
+                   |  n     n           n   |
+                   | D(f1) D(f2)  ...  D(fn)|
+
+    see: http://en.wikipedia.org/wiki/Wronskian
+    """
+
     for index in xrange(0, len(functions)):
-        functions[index] = Basic.sympify(functions[index])
+        functions[index] = sympify(functions[index])
     n = len(functions)
     W = Matrix(n, n, lambda i,j: functions[i].diff(var, j) )
     return W.det()
@@ -1028,7 +1212,7 @@ def casoratian(seqs, n, zero=True):
        True
 
     """
-    seqs = map(Basic.sympify, seqs)
+    seqs = map(sympify, seqs)
 
     if not zero:
         f = lambda i, j: seqs[j].subs(n, n+i)
@@ -1040,6 +1224,8 @@ def casoratian(seqs, n, zero=True):
     return Matrix(k, k, f).det()
 
 class SMatrix(Matrix):
+    """Sparse matrix"""
+
     def __init__(self, *args):
         if len(args) == 3 and callable(args[2]):
             op = args[2]
@@ -1049,7 +1235,7 @@ class SMatrix(Matrix):
             self.mat = {}
             for i in range(self.lines):
                 for j in range(self.cols):
-                    value = Basic.sympify(op(i,j))
+                    value = sympify(op(i,j))
                     if value != 0:
                         self.mat[(i,j)] = value
         elif len(args)==3 and isinstance(args[0],int) and \
@@ -1060,7 +1246,7 @@ class SMatrix(Matrix):
             self.mat = {}
             for i in range(self.lines):
                 for j in range(self.cols):
-                    value = Basic.sympify(mat[i*self.cols+j])
+                    value = sympify(mat[i*self.cols+j])
                     if value != 0:
                         self.mat[(i,j)] = value
         elif len(args)==3 and isinstance(args[0],int) and \
@@ -1084,7 +1270,7 @@ class SMatrix(Matrix):
             for i in range(self.lines):
                 assert len(mat[i]) == self.cols
                 for j in range(self.cols):
-                    value = Basic.sympify(mat[i][j])
+                    value = sympify(mat[i][j])
                     if value != 0:
                         self.mat[(i,j)] = value
 
@@ -1133,7 +1319,7 @@ class SMatrix(Matrix):
                 self.copyin_list(key, value)
         else:
             i,j=self.key2ij(key)
-            testval = Basic.sympify(value)
+            testval = sympify(value)
             if testval != 0:
                 self.mat[(i,j)] = testval
             elif self.mat.has_key((i,j)):
@@ -1238,3 +1424,20 @@ class SMatrix(Matrix):
             tmp[i,i] = 1
         return tmp
 
+
+def list2numpy(l):
+    """Converts python list of SymPy expressions to a NumPy array."""
+    from numpy import empty
+    a = empty(len(l), dtype=object)
+    for i, s in enumerate(l):
+        a[i] = s
+    return a
+
+def matrix2numpy(m):
+    """Converts SymPy's matrix to a NumPy array."""
+    from numpy import empty
+    a = empty(m.shape, dtype=object)
+    for i in range(m.lines):
+        for j in range(m.cols):
+            a[i, j] = m[i, j]
+    return a
