@@ -3,5 +3,5 @@
 from sympy.polynomials.base import PolynomialException, Polynomial
 from sympy.polynomials.roots_ import count_real_roots, roots, solve_system
 from sympy.polynomials.wrapper import div, quo, rem, factor, gcd, groebner, \
-     lcm, resultant, sqf, sqf_part
+     lcm, resultant, sqf, sqf_part, egcd
 #from sympy.polynomials.ideals import Ideal
