@@ -8,6 +8,7 @@ from sympy.core import numbers
 # To determine coefficient type:
 from sympy.core.numbers import NumberSymbol, ImaginaryUnit
 from sympy.utilities import *
+from sympy.core import sympify
 
 from sympy.polynomials import fast
 import sympy.polynomials    # for sympy.polynomials.wrapper  (cyclic)
@@ -184,6 +185,7 @@ class Polynomial(Basic):
                         "as_integer",
                         "as_monic",
                         "as_primitive",
+                        "coeff",
                         "content",
                         "diff",
                         "integrate",
@@ -657,7 +659,7 @@ class Polynomial(Basic):
             >>> f = Polynomial(6*x + 20*y + 4*x*y)
             >>> fx = f.integrate(x)
             >>> print fx
-            3*x**2 + 2*y*x**2 + 20*x*y
+            20*x*y + 3*x**2 + 2*y*x**2
             >>> fz = f.integrate(z)
             >>> print fz
             6*x*z + 20*y*z + 4*x*y*z
@@ -665,15 +667,33 @@ class Polynomial(Basic):
         """
 
         if not variable in self.var:
-            return (variable * self).expand()
-        nvar = self.var.index(variable) 
-        int = []
+            # FIXME we have to ensure first that either:
+            # - coeff_ring != 'sym'
+            # - coefficients do not contain `variable`
+            return Polynomial(variable, order=self.order) * self
+        nvar = self.var.index(variable)
+        cint = []
         for term in self.coeffs:
             t = list(term)
             t[nvar+1] += 1
             t[0] /= t[nvar+1]
-            int.append(tuple(t))
-        return coefficients2sympy(int, self.var) 
+            cint.append(tuple(t))
+
+        return Polynomial(coeffs=cint, var=self.var, order=self.order)
+
+    def coeff(self, x, n):
+        """Returns the coefficient at x**n
+
+        Example:
+        >>> a, x = symbols("ax")
+        >>> f = (a+1)*x + (a+2)*x**2 + a
+        >>> Polynomial(f).coeff(x, 2)
+        2 + a
+        >>> Polynomial(f).coeff(a, 1)
+        1 + x + x**2
+        """
+        f = Polynomial(self, var=[x])
+        return f.nth_coeff(n)
 
 
     def leading_coeff(self):
@@ -786,13 +806,13 @@ def sympy2coefficients(sympy_expr, var, order):
     """
 
     result_dict = {}
-    if isinstance(sympy_expr, Add):
-        terms = sympy_expr[:]
+    if sympy_expr.is_Add:
+        terms = sympy_expr.args[:]
     else:
         terms = [sympy_expr]
     for term in terms:
-        if isinstance(term, Mul):
-            factors = term[:]
+        if term.is_Mul:
+            factors = term.args[:]
         else:
             factors = [term]
         c = S.One
@@ -951,7 +971,7 @@ def coeff_ring(atom):
     if isinstance(atom, (Number, NumberSymbol, ImaginaryUnit)) \
         or (isinstance(atom, (Add, Mul))
             and all(map(lambda a:isinstance(a, (Number, NumberSymbol,
-                                                ImaginaryUnit)), atom[:]))) \
+                                                ImaginaryUnit)), atom.args))) \
         or (isinstance(atom, Pow) and isinstance(atom.base, (Number,
                                                              NumberSymbol,
                                                              ImaginaryUnit))
@@ -986,14 +1006,14 @@ def get_numbers(atom):
             and all(map(lambda a:isinstance(a, (Number,
                                                 NumberSymbol,
                                                 ImaginaryUnit)),
-                        atom[:]))) \
+                        atom.args[:]))) \
         or (isinstance(atom, Pow) and isinstance(atom.base, (Number,
                                                              NumberSymbol,
                                                              ImaginaryUnit))
             and isinstance(atom.exp, (Number, NumberSymbol, ImaginaryUnit))):
         return [atom]
     elif isinstance(atom, (Add, Mul)):
-        for a in atom:
+        for a in atom.args:
             result.append(get_numbers(a))
     return result
 
@@ -1016,7 +1036,7 @@ def Polynomial2IntPoly(f):
 
     Nothing is checked.
     """
-    
+
     result_dict = {}
     for t in f.coeffs:
         if t[0] is not S.Zero:
@@ -1030,7 +1050,7 @@ def IntPoly2Polynomial(f, var, order):
 
     Nothing is checked.
     """
-    
+
     coeffs = []
     exponents = f.coeffs.keys()
     exponents.sort(reverse=True)
@@ -1038,4 +1058,4 @@ def IntPoly2Polynomial(f, var, order):
         coeffs.append((sympify(f[exp]), sympify(exp)))
     return Polynomial(coeffs=tuple(coeffs), var=var, order=order)
 
-                      
+
