@@ -1,9 +1,9 @@
 
-from sympy.core import Basic, S, Add, Mul, Pow, Rational, Integer, \
-        Derivative, Wild, Symbol
+from sympy.core import Basic, S, C, Add, Mul, Pow, Rational, Integer, \
+        Derivative, Wild, Symbol, sympify
 
 from sympy.utilities import make_list, all
-from sympy.functions import gamma, exp
+from sympy.functions import gamma, exp, sqrt
 
 from sys import maxint
 
@@ -54,7 +54,7 @@ def fraction(expr, exact=False):
        (exp(-x), 1)
 
     """
-    expr = Basic.sympify(expr)
+    expr = sympify(expr)
 
     #XXX this only works sometimes (caching bug?)
     if expr == exp(-Symbol("x")) and exact:
@@ -63,34 +63,34 @@ def fraction(expr, exact=False):
     numer, denom = [], []
 
     for term in make_list(expr, Mul):
-        if isinstance(term, Pow):
+        if term.is_Pow:
             if term.exp.is_negative:
-                if term.exp == Integer(-1):
+                if term.exp is S.NegativeOne:
                     denom.append(term.base)
                 else:
                     denom.append(Pow(term.base, -term.exp))
-            elif not exact and isinstance(term.exp, Mul):
-                coeff, tail = term.exp[0], Mul(*term.exp[1:])#term.exp.getab()
+            elif not exact and term.exp.is_Mul:
+                coeff, tail = term.exp.args[0], Mul(*term.exp.args[1:])#term.exp.getab()
 
-                if isinstance(coeff, Rational) and coeff.is_negative:
+                if coeff.is_Rational and coeff.is_negative:
                     denom.append(Pow(term.base, -term.exp))
                 else:
                     numer.append(term)
             else:
                 numer.append(term)
-        elif isinstance(term, Basic.exp):
-            if term[0].is_negative:
-                denom.append(Basic.exp(-term[0]))
-            elif not exact and isinstance(term[0], Mul):
-                coeff, tail = term[0], Mul(*term[1:])#term.args.getab()
+        elif term.func is C.exp:
+            if term.args[0].is_negative:
+                denom.append(C.exp(-term.args[0]))
+            elif not exact and term.args[0].is_Mul:
+                coeff, tail = term.args[0], Mul(*term.args[1:])#term.args.getab()
 
-                if isinstance(coeff, Rational) and coeff.is_negative:
-                    denom.append(Basic.exp(-term[0]))
+                if coeff.is_Rational and coeff.is_negative:
+                    denom.append(C.exp(-term.args[0]))
                 else:
                     numer.append(term)
             else:
                 numer.append(term)
-        elif isinstance(term, Rational):
+        elif term.is_Rational:
             if term.is_integer:
                 numer.append(term)
             else:
@@ -135,9 +135,13 @@ def separate(expr, deep=False):
        >>> separate((x*sin(x))**y + (x*cos(x))**y)
        x**y*cos(x)**y + x**y*sin(x)**y
 
+       #this does not work because of exp combining
        #>>> separate((exp(x)*exp(y))**x)
        #exp(x*y)*exp(x**2)
 
+       >>> separate((sin(x)*cos(x))**y)
+       cos(x)**y*sin(x)**y
+
        Notice that summations are left un touched. If this is not the
        requested behaviour, apply 'expand' to input expression before:
 
@@ -148,26 +152,26 @@ def separate(expr, deep=False):
        x**(1 + z)*y**(1 + z)
 
     """
-    expr = Basic.sympify(expr)
+    expr = sympify(expr)
 
-    if isinstance(expr, Basic.Pow):
+    if expr.is_Pow:
         terms, expo = [], separate(expr.exp, deep)
         #print expr, terms, expo, expr.base
 
-        if isinstance(expr.base, Mul):
-            t = [ separate(Basic.Pow(t,expo), deep) for t in expr.base ]
-            return Basic.Mul(*t)
-        elif isinstance(expr.base, Basic.exp):
+        if expr.base.is_Mul:
+            t = [ separate(C.Pow(t,expo), deep) for t in expr.base.args ]
+            return C.Mul(*t)
+        elif expr.base.func is C.exp:
             if deep == True:
-                return Basic.exp(separate(expr.base[0], deep)*expo)
+                return C.exp(separate(expr.base[0], deep)*expo)
             else:
-                return Basic.exp(expr.base[0]*expo)
+                return C.exp(expr.base[0]*expo)
         else:
-            return Basic.Pow(separate(expr.base, deep), expo)
-    elif isinstance(expr, (Basic.Add, Basic.Mul)):
-        return type(expr)(*[ separate(t, deep) for t in expr ])
-    elif isinstance(expr, Basic.Function) and deep:
-        return expr.func(*[ separate(t) for t in expr])
+            return C.Pow(separate(expr.base, deep), expo)
+    elif expr.is_Add or expr.is_Mul:
+        return type(expr)(*[ separate(t, deep) for t in expr.args ])
+    elif expr.is_Function and deep:
+        return expr.func(*[ separate(t) for t in expr.args])
     else:
         return expr
 
@@ -228,38 +232,38 @@ def together(expr, deep=False):
 
         from sympy.core.function import Function
 
-        if isinstance(expr, Add):
+        if expr.is_Add:
             items, coeffs, basis = [], [], {}
 
-            for elem in expr:
+            for elem in expr.args:
                 numer, q = fraction(_together(elem))
 
                 denom = {}
 
                 for term in make_list(q.expand(), Mul):
-                    expo = Integer(1)
-                    coeff = Integer(1)
+                    expo = S.One
+                    coeff = S.One
 
 
-                    if isinstance(term, Pow):
-                        if isinstance(term.exp, Rational):
+                    if term.is_Pow:
+                        if term.exp.is_Rational:
                             term, expo = term.base, term.exp
-                        elif isinstance(term.exp, Mul):
+                        elif term.exp.is_Mul:
                             coeff, tail = term.exp.as_coeff_terms()
-                            if isinstance(coeff, Rational):
-                                tail = Basic.Mul(*tail)
+                            if coeff.is_Rational:
+                                tail = C.Mul(*tail)
                                 term, expo = Pow(term.base, tail), coeff
-                        coeff = Integer(1)
-                    elif isinstance(term, Basic.exp):
-                        if isinstance(term[0], Rational):
-                            term, expo = Basic.E, term[0]
-                        elif isinstance(term[0], Mul):
+                        coeff = S.One
+                    elif term.func is C.exp:
+                        if term[0].is_Rational:
+                            term, expo = C.E, term[0]
+                        elif term[0].is_Mul:
                             coeff, tail = term[0].as_coeff_terms()
-                            if isinstance(coeff, Rational):
-                                tail = Basic.Mul(*tail)
-                                term, expo = Basic.exp(tail), coeff
-                        coeff = Integer(1)
-                    elif isinstance(term, Rational):
+                            if coeff.is_Rational:
+                                tail = C.Mul(*tail)
+                                term, expo = C.exp(tail), coeff
+                        coeff = S.One
+                    elif term.is_Rational:
                         coeff = Integer(term.q)
                         term = Integer(term.p)
 
@@ -286,10 +290,10 @@ def together(expr, deep=False):
             for (term, (total, maxi)) in basis.iteritems():
                 basis[term] = (total, total-maxi)
 
-                if isinstance(term, Basic.exp):
-                    denominator.append(Basic.exp(maxi*term[:]))
+                if term.func is C.exp:
+                    denominator.append(C.exp(maxi*term[:]))
                 else:
-                    if isinstance(maxi, Basic.One):
+                    if maxi is S.One:
                         denominator.append(term)
                     else:
                         denominator.append(Pow(term, maxi))
@@ -300,7 +304,7 @@ def together(expr, deep=False):
                 gcds = lambda x, y: int_gcd(int(x), int(y))
                 common = Rational(reduce(gcds, coeffs))
             else:
-                common = Rational(1)
+                common = S.One
 
             product = reduce(lambda x, y: x*y, coeffs) / common
 
@@ -316,10 +320,10 @@ def together(expr, deep=False):
                     else:
                         expo = total-sub
 
-                    if isinstance(term, Basic.exp):
-                        expr.append(Basic.exp(expo*term[:]))
+                    if term.func is C.exp:
+                        expr.append(C.exp(expo*term[:]))
                     else:
-                        if isinstance(expo, Basic.One):
+                        if expo is S.One:
                             expr.append(term)
                         else:
                             expr.append(Pow(term, expo))
@@ -327,10 +331,10 @@ def together(expr, deep=False):
                 numerator.append(coeff*Mul(*([numer] + expr)))
 
             return Add(*numerator)/(product*Mul(*denominator))
-        elif isinstance(expr, (Mul, Pow)):
-            return type(expr)(*[ _together(t) for t in expr ])
-        elif isinstance(expr, Function) and deep:
-            return expr.func(*[ _together(t) for t in expr ])
+        elif expr.is_Mul or expr.is_Pow:
+            return type(expr)(*[ _together(t) for t in expr.args ])
+        elif expr.is_Function and deep:
+            return expr.func(*[ _together(t) for t in expr.args ])
         else:
             return expr
 
@@ -371,8 +375,13 @@ def collect(expr, syms, evaluate=True, exact=False):
 
        The same result can achieved in dictionary form:
 
-       >>> collect(a*x**2 + b*x**2 + a*x - b*x + c, x, evaluate=False)
-       {1: c, x**2: a + b, x: a - b}
+       >>> d = collect(a*x**2 + b*x**2 + a*x - b*x + c, x, evaluate=False)
+       >>> d[x**2]
+       a + b
+       >>> d[x]
+       a - b
+       >>> d[sympify(1)]
+       c
 
        You can also work with multi-variate polynomials. However
        remember that this function is greedy so it will care only
@@ -459,7 +468,7 @@ def collect(expr, syms, evaluate=True, exact=False):
                     term, order = Derivative(term, var), order-1
 
             if sym is None:
-                if isinstance(rat, Basic.One):
+                if rat is S.One:
                     product.append(term)
                 else:
                     product.append(Pow(term, rat))
@@ -479,34 +488,34 @@ def collect(expr, syms, evaluate=True, exact=False):
         return expr, (sym, Rational(order))
 
     def parse_term(expr):
-        rat_expo, sym_expo = Rational(1), None
+        rat_expo, sym_expo = S.One, None
         sexpr, deriv = expr, None
 
-        if isinstance(expr, Pow):
+        if expr.is_Pow:
             if isinstance(expr.base, Derivative):
                 sexpr, deriv = parse_derivative(expr.base)
             else:
                 sexpr = expr.base
 
-            if isinstance(expr.exp, Rational):
+            if expr.exp.is_Rational:
                 rat_expo = expr.exp
-            elif isinstance(expr.exp, Mul):
+            elif expr.exp.is_Mul:
                 coeff, tail = term.exp.as_coeff_terms()
 
-                if isinstance(coeff, Rational):
-                    rat_expo, sym_expo = coeff, Basic.Mul(*tail)
+                if coeff.is_Rational:
+                    rat_expo, sym_expo = coeff, C.Mul(*tail)
                 else:
                     sym_expo = expr.exp
             else:
                 sym_expo = expr.exp
-        elif isinstance(expr, Basic.exp):
-            if isinstance(expr[0], Rational):
-                sexpr, rat_expo = Basic.exp(Rational(1)), expr[0]
-            elif isinstance(expr[0], Mul):
-                coeff, tail = expr[0].as_coeff_terms()
+        elif expr.func is C.exp:
+            if expr.args[0].is_Rational:
+                sexpr, rat_expo = S.Exp1, expr[0]
+            elif expr.args[0].is_Mul:
+                coeff, tail = expr.args[0].as_coeff_terms()
 
-                if isinstance(coeff, Rational):
-                    sexpr, rat_expo = Basic.exp(Basic.Mul(*tail)), coeff
+                if coeff.is_Rational:
+                    sexpr, rat_expo = C.exp(C.Mul(*tail)), coeff
         elif isinstance(expr, Derivative):
             sexpr, deriv = parse_derivative(expr)
 
@@ -522,7 +531,7 @@ def collect(expr, syms, evaluate=True, exact=False):
         else:
             pattern = [ parse_term(elem) for elem in pattern ]
 
-            elems, common_expo, has_deriv = [], Rational(1), False
+            elems, common_expo, has_deriv = [], S.One, False
 
             for elem, e_rat, e_sym, e_ord in pattern:
                 if e_ord is not None:
@@ -539,7 +548,7 @@ def collect(expr, syms, evaluate=True, exact=False):
                             # for both expression's term and pattern's element
                             expo = t_rat / e_rat
 
-                            if isinstance(common_expo, Basic.One):
+                            if common_expo is S.One:
                                 common_expo = expo
                             else:
                                 # common exponent was negotiated before so
@@ -566,23 +575,23 @@ def collect(expr, syms, evaluate=True, exact=False):
             return terms, elems, common_expo, has_deriv
 
     if evaluate:
-        if isinstance(expr, Basic.Mul):
+        if expr.is_Mul:
             ret = 1
-            for term in expr:
+            for term in expr.args:
                 ret *= collect(term, syms, True, exact)
             return ret
-        elif isinstance(expr, Basic.Pow):
+        elif expr.is_Pow:
             b = collect(expr.base, syms, True, exact)
-            return Basic.Pow(b, expr.exp)
+            return C.Pow(b, expr.exp)
 
-    summa = [ separate(i) for i in make_list(Basic.sympify(expr), Add) ]
+    summa = [ separate(i) for i in make_list(sympify(expr), Add) ]
 
     if isinstance(syms, list):
         syms = [ separate(s) for s in syms ]
     else:
         syms = [ separate(syms) ]
 
-    collected, disliked = {}, Rational(0)
+    collected, disliked = {}, S.Zero
 
     for product in summa:
         terms = [ parse_term(i) for i in make_list(product, Mul) ]
@@ -613,8 +622,8 @@ def collect(expr, syms, evaluate=True, exact=False):
             # none of the patterns matched
             disliked += product
 
-    if disliked != Rational(0):
-        collected[Rational(1)] = disliked
+    if disliked is not S.Zero:
+        collected[S.One] = disliked
 
     if evaluate:
         return Add(*[ a*b for a, b in collected.iteritems() ])
@@ -639,20 +648,20 @@ def ratsimp(expr):
         >>> y = Symbol('y')
         >>> e = ratsimp(1/x + 1/y)
     """
-
-    if isinstance(expr, Pow):
+    expr = sympify(expr)
+    if expr.is_Pow:
         return Pow(ratsimp(expr.base), ratsimp(expr.exp))
-    elif isinstance(expr, Mul):
+    elif expr.is_Mul:
         res = []
-        for x in expr:
+        for x in expr.args:
             res.append( ratsimp(x) )
         return Mul(*res)
-    elif isinstance(expr, Basic.Function):
-        return expr.func(*[ ratsimp(t) for t in expr ])
+    elif expr.is_Function:
+        return expr.func(*[ ratsimp(t) for t in expr.args ])
 
-    #elif isinstance(expr, Function):
+    #elif expr.is_Function:
     #    return type(expr)( ratsimp(expr[0]) )
-    elif not isinstance(expr, Add):
+    elif not expr.is_Add:
         return expr
 
     def get_num_denum(x):
@@ -661,10 +670,10 @@ def ratsimp(expr):
         r = x.match(a/b)
         if r is not None and len(r) == 2:
             return r[a],r[b]
-        return x, 1
+        return x, S.One
 
-    x = expr[0]
-    y = Add(*expr[1:])
+    x = expr.args[0]
+    y = Add(*expr.args[1:])
 
     a,b = get_num_denum(ratsimp(x))
     c,d = get_num_denum(ratsimp(y))
@@ -680,12 +689,12 @@ def ratsimp(expr):
     #but SymPy doesn't yet have a multivariate polynomial factorisation
     #so until we have it, we are just returning the correct results here
     #to pass all tests...
-    if isinstance(denum,Pow):
-        e = (num/denum[0]).expand()
+    if denum.is_Pow:
+        e = (num/denum.args[0]).expand()
         f = (e/(-2*Symbol("y"))).expand()
-        if f == denum/denum[0]:
+        if f == denum/denum.args[0]:
             return -2*Symbol("y")
-        return e/(denum/denum[0])
+        return e/(denum/denum.args[0])
     return num/denum
 
 def trigsimp(expr, deep=False):
@@ -712,23 +721,23 @@ def trigsimp(expr, deep=False):
         log(2)
     """
     from sympy.core.basic import S
-    sin, cos, tan, cot = Basic.sin, Basic.cos, Basic.tan, Basic.cot
+    sin, cos, tan, cot = C.sin, C.cos, C.tan, C.cot
 
     #XXX this stopped working:
     if expr == 1/cos(Symbol("x"))**2 - 1:
         return tan(Symbol("x"))**2
 
-    if isinstance(expr, Basic.Function):
+    if expr.is_Function:
         if deep:
-            return expr.func( trigsimp(expr[0], deep) )
-    elif isinstance(expr, Mul):
-        ret = Rational(1)
-        for x in expr:
+            return expr.func( trigsimp(expr.args[0], deep) )
+    elif expr.is_Mul:
+        ret = S.One
+        for x in expr.args:
             ret *= trigsimp(x, deep)
         return ret
-    elif isinstance(expr, Pow):
+    elif expr.is_Pow:
         return Pow(trigsimp(expr.base, deep), trigsimp(expr.exp, deep))
-    elif isinstance(expr, Add):
+    elif expr.is_Add:
         # TODO this needs to be faster
 
         # The types of trig functions we are looking for
@@ -740,14 +749,14 @@ def trigsimp(expr, deep=False):
         )
 
         # Scan for the terms we need
-        ret = Integer(0)
-        for term in expr:
+        ret = S.Zero
+        for term in expr.args:
             term = trigsimp(term, deep)
             res = None
             for pattern, result in matchers:
                 res = term.match(pattern)
                 if res is not None:
-                    ret += result.subs_dict(res)
+                    ret += result.subs(res)
                     break
             if res is None:
                 ret += term
@@ -771,9 +780,9 @@ def trigsimp(expr, deep=False):
 
             m = expr.match(pattern)
             while m is not None:
-                if m[a_t] == 0 or -m[a_t] in m[c][:] or m[a_t] + m[c] == 0:
+                if m[a_t] == 0 or -m[a_t] in m[c].args or m[a_t] + m[c] == 0:
                     break
-                expr = result.subs_dict(m)
+                expr = result.subs(m)
                 m = expr.match(pattern)
 
         return expr
@@ -795,7 +804,7 @@ def radsimp(expr):
     """
     n,d = fraction(expr)
     a,b,c = map(Wild, 'abc')
-    r = d.match(a+b*Basic.sqrt(c))
+    r = d.match(a+b*sqrt(c))
     if r is not None:
         a = r[a]
         if r[b] == 0:
@@ -803,8 +812,8 @@ def radsimp(expr):
         else:
             b,c = r[b],r[c]
 
-        syms = list(n.atoms(type=Basic.Symbol))
-        n = collect( (n*(a-b*Basic.sqrt(c))).expand(), syms )
+        syms = list(n.atoms(type=C.Symbol))
+        n = collect( (n*(a-b*sqrt(c))).expand(), syms )
         d = a**2 - c*b**2
 
     return n/d
@@ -837,20 +846,20 @@ def powsimp(expr, deep=False):
         log(n**(1 - n))
     """
     def _powsimp(expr):
-        if isinstance(expr, Basic.Pow):
+        if expr.is_Pow:
             if deep:
-                return Basic.Pow(powsimp(expr.base), powsimp(expr.exp))
+                return C.Pow(powsimp(expr.base), powsimp(expr.exp))
             return expr
-        #elif isinstance(expr, Basic.Apply) and deep:
-        #    return expr.func(*[powsimp(t) for t in expr])
-        elif isinstance(expr, Basic.Add):
-            return Basic.Add(*[powsimp(t) for t in expr])
-        elif isinstance(expr, Basic.Mul):
+        elif expr.is_Function and deep:
+            return expr.func(*[powsimp(t) for t in expr.args])
+        elif expr.is_Add:
+            return C.Add(*[powsimp(t) for t in expr.args])
+        elif expr.is_Mul:
             # Collect base/exp data, while maintaining order in the
             # non-commutative parts of the product
             c_powers = {}
             nc_part = []
-            for term in expr:
+            for term in expr.args:
                 if term.is_commutative:
                     b,e = term.as_base_exp()
                     c_powers[b] = c_powers.get(b, 0) + e
@@ -860,13 +869,13 @@ def powsimp(expr, deep=False):
             # Pull out numerical coefficients from exponent
             for b,e in c_powers.items():
                 exp_c, exp_t = e.as_coeff_terms()
-                if not isinstance(exp_c, Basic.One) and exp_t:
+                if not (exp_c is S.One) and exp_t:
                     del c_powers[b]
-                    new_base = Basic.Pow(b, exp_c)
+                    new_base = C.Pow(b, exp_c)
                     if new_base in c_powers:
-                        c_powers[new_base] += Basic.Mul(*exp_t)
+                        c_powers[new_base] += C.Mul(*exp_t)
                     else:
-                        c_powers[new_base] = Basic.Mul(*exp_t)
+                        c_powers[new_base] = C.Mul(*exp_t)
 
             # Combine bases whenever they have the same exponent which is
             # not numeric
@@ -889,26 +898,12 @@ def powsimp(expr, deep=False):
                     else:
                         c_powers[new_base] = e
 
-            c_part = [ Basic.Pow(b,e) for b,e in c_powers.items() ]
-            return Basic.Mul(*(c_part + nc_part))
+            c_part = [ C.Pow(b,e) for b,e in c_powers.items() ]
+            return C.Mul(*(c_part + nc_part))
         return expr
 
     return _powsimp(separate(expr, deep=deep))
 
-def normal(expr, *syms):
-    p, q = together(expr).as_numer_denom()
-
-    if p.is_polynomial(*syms) and q.is_polynomial(*syms):
-        from sympy.polynomials import gcd, quo
-
-        G = gcd(p, q, syms)
-
-        if not isinstance(G, Basic.One):
-            p = quo(p, G, syms)
-            q = quo(q, G, syms)
-
-    return p / q
-
 def hypersimp(term, n, consecutive=True, simplify=True):
     """Given combinatorial term a(n) simplify its consecutive term
        ratio ie. a(n+1)/a(n). The term can be composed of functions
@@ -947,7 +942,7 @@ def hypersimp(term, n, consecutive=True, simplify=True):
        [1] W. Koepf, Algorithms for m-fold Hypergeometric Summation,
            Journal of Symbolic Computation (1995) 20, 399-417
     """
-    term = Basic.sympify(term)
+    term = sympify(term)
 
     if consecutive == True:
         term = term.subs(n, n+1)/term
@@ -962,7 +957,7 @@ def hypersimp(term, n, consecutive=True, simplify=True):
 
             G = gcd(p, q, n)
 
-            if not isinstance(G, Basic.One):
+            if G is not S.One:
                 p = quo(p, G, n)
                 q = quo(q, G, n)
 
@@ -988,31 +983,20 @@ def combsimp(expr):
     return expr
 
 def simplify(expr):
-    """
-    Simplifies the expression 'expr'. 
+    """Naively simplifies the given expression.
+
+       Simplification is not a well defined term and the exact strategies
+       this function tries can change in the future versions of SymPy. If
+       your algorithm relies on "simplification" (whatever it is), try to
+       determine what you need exactly  -  is it powsimp(), or radsimp(),
+       or together(), or something else? And use this particular function
+       directly, because those are well defined and thus your algorithm
+       will be robust.
 
-    Simplification is not a well defined term and the exact strategies this
-    function tries can change in the future versions of SymPy. If your
-    algorithm relies on "simplification" (whatever it is), try to determine
-    what you need exactly - is it powsimp(), or radsimp(), or together(), or
-    something else? And use this particular function directly, because those
-    are well defined and thus your algorithm will be robust.
     """
-    #from sympy.specfun.factorials import factorial, factorial_simplify
-    #if expr.has_class(factorial):
-    #    expr = factorial_simplify(expr)
-    a,b = [ t.expand() for t in fraction(powsimp(expr)) ]
-    ret = together(radsimp(ratsimp(a/b)))
-    n,d = fraction(ret)
-    if isinstance(d, (Basic.One, Basic.NegativeOne)):
-        return d*n
-    n_var = n.atoms(type=Symbol)
-    d_var = d.atoms(type=Symbol)
-    if n_var and d_var and n.is_polynomial(*n_var) and d.is_polynomial(*d_var):
-        from sympy.polynomials import div, factor
-        q,r = div(n, d)
-        if r == 0:
-            return q
-        else:
-            return q + factor(r) / factor(d)
-    return ret
+    from sympy.polys import Poly
+
+    expr = powsimp(radsimp(expr))
+    expr = Poly._cancel(expr)
+
+    return together(expr.expand())
